


// ============================================================================================
// Libs
// ============================================================================================

//#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <mmsystem.h>

#include "zed4d.h"
#include "zhelp.h"
#include "zarray.h"
#include "zstring.h"
#include "zconsole.h"
#include "main.h"

// --------------------------------------------------------------------------------------------


// ============================================================================================
// Variablen
// ============================================================================================

HWND           g_hMainWnd;
HINSTANCE      g_hMainInstance;
BOOL           g_bWindowActive = false;
BOOL           g_bGamedown = false;
BOOL           g_bGameGo = false;

zD3DManager    Manager;
zD3DCamera     Cam;
zD3DFont       Font;
zTimer         Timer;

zConsole       Con;

DWORD          backgroundcolor=0xffff00ff;
bool           showfps = true;

// --------------------------------------------------------------------------------------------


// ============================================================================================
// Funktionen
// ============================================================================================

bool enumfunc(D3DCAPS8* pddDriverCaps, DWORD dw, D3DFORMAT d3dfrmt)
{
   return true;
}

bool Game_Init(VOID *Data)
{
   _write("Init starting...\n");

   if (!Manager.Init(g_hMainWnd,g_hMainInstance,WIDTH,HEIGHT,BPP,ZPP,SPP,false,false,&enumfunc)) {
      OutputDebugString("Manager.Init failed\n");
      return false;
   }

   // Projektionsmatrix: View Frustum 45 Grad
   D3DXMATRIX matrix;
   D3DXMatrixPerspectiveFovLH(&matrix, 0.8f, 4.0f / 3.0f, 1.0f, 1300.0f);
   _setts(D3DTS_PROJECTION, &matrix);

   Font.Init("Arial",12);
   Font.Restore();

   _setrs(D3DRS_LIGHTING,false);

   Con.Init(WIDTH,HEIGHT/2-10,"console.dds");

   g_bGameGo = true;
   _write("Init succeded\n");
   return true;
}

// --------------------------------------------------------------------------------------------

bool Game_Main(VOID *Data)
{
   char szfps[20];

   Timer.Frame();
   _gcvt(Timer.GetAverageFramesPerSec(),6,szfps);

   _httest(_clear(D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,backgroundcolor,1.0f,0),
           "clear failed\n");

   _httest(_beginscene(), "BeginScene failed\n");

   Con.Render();

   if (showfps) {
      _bttest(Font.DrawText(5,5,0xffffffff,szfps),
              "DrawText failed\n");
   }

   _httest(_endscene(), "EndScene failed\n");

   _httest(_present(),
           "present failed\n");

   return true;
}


// --------------------------------------------------------------------------------------------

bool Game_Shutdown(VOID *Data)
{
   EM.Write("Shutdown starting...\n");

   Con.Shutdown();
   Font.Restore();
   Font.Shutdown();
   Manager.Shutdown();

   g_bGameGo = false;
   EM.Write("Shutdown succeded\n");
   return true;
}

// --------------------------------------------------------------------------------------------

LRESULT CALLBACK WinProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
   switch (msg) {
      case WM_ACTIVATEAPP:
         g_bWindowActive = (bool) wParam;
        return 0;
      case WM_DESTROY:
         g_bGamedown = true;
         PostQuitMessage(0);
        return 0;
      case WM_KEYDOWN:
         if (wParam == VK_F1) {
            Con.ChangeState();
            return 0;
         }
         if (wParam == VK_ESCAPE) {
            PostMessage(hWnd,WM_CLOSE,0,0);
            return 0;
         }
         if (Con.IsActive())
            Con.Input(wParam);
        break;
   }
   return DefWindowProc(hWnd,msg,wParam,lParam);
}

// --------------------------------------------------------------------------------------------

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
   WNDCLASSEX     winclass;
   MSG            msg;
   HWND           hWnd;

   winclass.cbSize            = sizeof(WNDCLASSEX);
   winclass.style             = CS_HREDRAW | CS_VREDRAW;
   winclass.lpfnWndProc       = WinProc;
   winclass.cbClsExtra        = 0;
   winclass.cbWndExtra        = 0;
   winclass.hInstance         = hInstance;
   winclass.hIcon             = LoadIcon(NULL, IDI_APPLICATION);
   winclass.hCursor           = LoadCursor(NULL, IDC_ARROW); 
   winclass.hbrBackground     = (HBRUSH)GetStockObject(BLACK_BRUSH);
   winclass.lpszMenuName      = NULL;
   winclass.lpszClassName     = WINDOW_CLASS_NAME;
   winclass.hIconSm           = LoadIcon(NULL, IDI_APPLICATION);

   if (!RegisterClassEx(&winclass))
      return -1;

   if (!(hWnd = CreateWindowEx(  WS_EX_TOPMOST,
                                 WINDOW_CLASS_NAME,
                                 WINDOW_TITLE,
                                 WS_POPUP | WS_VISIBLE,
                                 0,0,
                                 GetSystemMetrics(SM_CXSCREEN),GetSystemMetrics(SM_CYSCREEN),
                                 NULL,
                                 NULL,
                                 hInstance,
                                 NULL)))
      return -2;

   g_hMainWnd = hWnd;
   g_hMainInstance = hInstance;

   UpdateWindow(hWnd);

   if (!Game_Init(NULL))
      return -3;

   while (!g_bGamedown) {
      if (PeekMessage(&msg,hWnd,0,0,PM_REMOVE)) {
         TranslateMessage(&msg);
         DispatchMessage(&msg);
      }

      if (g_bWindowActive && g_bGameGo) {
         if (!Game_Main(NULL)) {
            g_bGameGo = false;
            PostMessage(hWnd,WM_CLOSE,0,0);
         }
      }
   }

   Game_Shutdown(NULL);
   return msg.wParam;
}
