/* ============================================================================================= *
 * file:    console.h
 * created: 2002-03-18
 * coder:   [kon]
 * ============================================================================================= */

/*************************************************************************************************/
/*************************************************************************************************/
// ***** Includes
//#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdlib.h>
#include "d3d8.h"
#include "d3dx8.h"
#include "zed4d.h"
#include "zhelp.h"
#include "zstring.h"
#include "zarray.h"
#include "zconsole.h"
#include "zconsole_cmds.h"


/************************************* class zConsoleCommand *************************************/


/*************************************************************************************************/
/*************************************************************************************************/
// ***** command funcs

zConsoleCommand::zConsoleCommand()
{
   SetCommandString(""); SetParamString(""); SetHelpString(""); mpFunc=NULL; mVars=NULL;
}

/*************************************************************************************************/

zConsoleCommand::~zConsoleCommand()
{
   if (mVars == NULL) return;
   for (int i=0; i < miNumVars; i++) {
      _deletearray(mVars[i].s);
   }
   miNumVars = 0;
   _deletearray(mVars);
}

/*************************************************************************************************/

zConsoleCommand::zConsoleCommand(zString command, zString paramCodes, zString help)
{
   SetCommandString(command); SetParamString(paramCodes); SetHelpString(help);
   mpFunc=NULL; mVars=NULL;
}

/*************************************************************************************************/

EConMsgParamID zConsoleCommand::GetParamType(int i) const
{
   if (strlen(mszParamString) <= i && i>=0) return CON_PARAM_UNKNOWN;

   switch (toupper(mszParamString[i]))
   {
      case 'S': return CON_PARAM_STRING; break;
      case 'I': return CON_PARAM_INT;    break;
      case 'F': return CON_PARAM_FLOAT;  break;
      case 'B': return CON_PARAM_BOOL;   break;

      default: _write("zConsolecommand:GetParamType -> UNKNOWN\n"); return CON_PARAM_UNKNOWN;
   }
}

/*************************************************************************************************/

bool zConsoleCommand::HasValidParams()
{
   for (int i=0; i < GetNumParams(); i++) {
      switch(GetParamType(i)) {
         case CON_PARAM_UNKNOWN: return false; break;
         default:;
      }
   }
   return true;
}

/*************************************************************************************************/

// Erzeugt aus einem String einen Bool (true/false oder 1/0)
bool getbool(zString param)
{
   if (strlen(param) == 1) {
      if (!strcmp(param,"0"))
         return false;
      if (!strcmp(param,"1"))
         return true;
   }
   else {
      if (!strcmp(param,"true"))
         return true;
      if (!strcmp(param,"false"))
         return false;
   }
   return false;
}

/*************************************************************************************************/

void zConsoleCommand::CreateVarsFromStrings(zArray<zString>* params)
{
   if (params == NULL) return;
   if (params->GetNum() == 0) return;
   if (params->GetNum() != GetNumParams()) return;
   // Array noch nicht angelegt? Dann aber schnell...
   if (miNumVars != GetNumParams()) {
      mVars = (zConVars*) realloc(mVars,sizeof(zConVars)*GetNumParams());
      miNumVars = GetNumParams();
   }

   // Variablen in Array speichern
   for (int i=0; i < params->GetNum(); i++) {
      switch (GetParamType(i)) {
         case CON_PARAM_INT:    mVars[i].i = atoi((*params)[i]);     break;
         case CON_PARAM_FLOAT:  mVars[i].f = atof((*params)[i]);     break;
         case CON_PARAM_BOOL:   mVars[i].b = getbool((*params)[i]);  break;
         case CON_PARAM_STRING: _deletearray(mVars[i].s); mVars[i].s = new char[strlen((*params)[i])+1]; strcpy(mVars[i].s,(*params)[i]);  break;

         default: _write("Console : *warning* using unknown type in console message conversion!\n");
      }
   }
}

/*************************************************************************************************/


/**************************************** class zConsole *****************************************/


/*************************************************************************************************/
/*************************************************************************************************/
// ***** private funcs

void zConsole::PreInit()
{
   // Stanard Initialisierung
   mfWidth = 0.0f;
   mfHeight = 0.0f;
   mszTexfile = "";
   mLogfile = "console-debug.txt";
   mbLogging = false;
   mpTexture = NULL;
   mbIsValidTexture = false;
   mdwDiffuse = VERTEX_DIFFUSE;
   mdwStateBlock = 0xffffffff;
   mdwSavedBlock = 0xffffffff;
   mState = CON_STATE;
   miInputPos = -1;
   mszUsedFont = "";
   miFontHeight = 0;
   mdwFontLineColor = FONT_LINE_COLOR;
   mdwFontHistoryColor = FONT_HISTORY_COLOR;
   miNumCommands = 0;
   miIndCommands = 0;
   for (int i=0; i < MAX_COMMAND_STRINGS; i++)
      for (int e=0; e < MAX_STR_LEN; e++)
         maCommand[i][e] = '\0';
   miShowCommand = 0;
   for (i=0; i < MAX_HISTORY_STRINGS; i++)
      for (int e=0; e < MAX_HISTORY_STR_LEN; e++)
         maHistory[i][e] = '\0';
   miNumHistoryEntries = 0;
   miIndHistory = 0;
   miHistoryOffset = 0;
   mpFunc = NULL;
   for (i=0; i < MAX_STR_LEN; i++) mInput[i] = '\0';
   verts[0].x = 0.0f;   verts[0].y = 0.0f;  verts[0].z = 0.0f;
   verts[0].rhw = 1.0f; verts[0].tu = 0.0f; verts[0].tv = 0.0f;
   verts[1].x = 0.0f;   verts[1].y = 0.0f;  verts[1].z = 0.0f;
   verts[1].rhw = 1.0f; verts[1].tu = 0.0f; verts[1].tv = 0.0f;
   verts[2].x = 0.0f;   verts[2].y = 0.0f;  verts[2].z = 0.0f;
   verts[2].rhw = 1.0f; verts[2].tu = 0.0f; verts[2].tv = 0.0f;
   verts[3].x = 0.0f;   verts[3].y = 0.0f;  verts[3].z = 0.0f;
   verts[3].rhw = 1.0f; verts[3].tu = 0.0f; verts[3].tv = 0.0f;
}

/*************************************************************************************************/

bool zConsole::CreateStateBlocks()
{
   for (int jj = 0; jj < 2; ++jj) {
      _httest(pDev->BeginStateBlock(),
              "zConsole.CreateStateBlocks: BeginStateBlock failed\n");
      pDev->SetRenderState( D3DRS_ALPHABLENDENABLE,         TRUE );
      pDev->SetRenderState( D3DRS_SRCBLEND,                 D3DBLEND_SRCALPHA );
      pDev->SetRenderState( D3DRS_DESTBLEND,                D3DBLEND_INVSRCALPHA );
      pDev->SetRenderState( D3DRS_ALPHATESTENABLE,          TRUE );
      pDev->SetRenderState( D3DRS_CULLMODE,                 D3DCULL_CCW );
      pDev->SetRenderState( D3DRS_ZENABLE,                  FALSE );
      pDev->SetRenderState( D3DRS_STENCILENABLE,            FALSE );
      pDev->SetRenderState( D3DRS_CLIPPING,                 TRUE );
      pDev->SetRenderState( D3DRS_EDGEANTIALIAS,            FALSE );
      pDev->SetRenderState( D3DRS_FOGENABLE,                FALSE );
      pDev->SetTextureStageState( 0, D3DTSS_ALPHAARG1,      D3DTA_DIFFUSE );
      pDev->SetTextureStageState( 0, D3DTSS_COLOROP,        D3DTOP_MODULATE );
      pDev->SetTextureStageState( 0, D3DTSS_COLORARG1,      D3DTA_TEXTURE );
      pDev->SetTextureStageState( 0, D3DTSS_COLORARG2,      D3DTA_DIFFUSE );
      if (jj == 0) {
         _httest(pDev->EndStateBlock(&mdwStateBlock),
                 "zConsole.CreateStateBlocks: EndStateBlock failed\n");
      }
      else {
         _httest(pDev->EndStateBlock(&mdwSavedBlock),
                 "zConsole.CreateStateBlocks: EndStateBlock failed\n");
      }
   }
   
   return true;
}

/*************************************************************************************************/

void zConsole::AddLineToCommandHistory()
{
   ++miNumCommands;
   ++miIndCommands;

   // Maximale Anzahl von CommandStrings in History
   if (miNumCommands > MAX_COMMAND_STRINGS)
      miNumCommands = MAX_COMMAND_STRINGS;

   // Ende des Arrays? Wieder von vorne beginnen
   if (miIndCommands > MAX_COMMAND_STRINGS)
      miIndCommands = 1;

   // Input kopieren
   strcpy(maCommand[miIndCommands-1],mInput);

   // letztes Command muss angezeigt werden
   miShowCommand = miIndCommands-1;

   // Log?
   if (mbLogging) {
      FILE *file = fopen(mLogfile,"a");
      fwrite("con_com: ",sizeof(char), 9, file);
      fwrite(mInput, sizeof(char), strlen(mInput), file);
      fwrite("\n", sizeof(char), 1, file);
      fclose(file);
   }
}

/*************************************************************************************************/

void zConsole::AddLineToHistory(zString& line)
{
   ++miNumHistoryEntries;
   ++miIndHistory;

   if (miNumHistoryEntries > MAX_HISTORY_STRINGS)
      miNumHistoryEntries = MAX_HISTORY_STRINGS;

   if (miIndHistory > MAX_HISTORY_STRINGS)
      miIndHistory = 1;

   strcpy(maHistory[miIndHistory-1],line);

   // Log?
   if (mbLogging) {
      FILE *file = fopen(mLogfile,"a");
      fwrite("con_his: ",sizeof(char), 9, file);
      fwrite(line, sizeof(char), strlen(line), file);
      fwrite("\n", sizeof(char), 1, file);
      fclose(file);
   }
}

/*************************************************************************************************/

void zConsole::TryCommandComplete()
{
   int ind=-1;

   // Alle mglichen Commands suchen, die passen
   for (int i=0; i < mpCommands.GetNum(); i++) {
      if ((strlen(mpCommands[i].GetCommandString()) > strlen(mInput)) &&!strncmp(mInput,mpCommands[i].GetCommandString(),strlen(mInput))) {
         if (ind != -1)   // Mehr als Eins! -> Abbruch, da nicht sicher, welches der Benutzer meint
            return;
         ind = i;
      }
   }

   // Nur ein Command Match gefunden, in mInput speichern
   if (ind != -1) {
      strcpy(mInput, mpCommands[ind].GetCommandString());
      mInput[strlen(mInput)] = ' ';
      miInputPos = strlen(mInput)-1;
   }
}

/*************************************************************************************************/

void zConsole::InitCommands()
{
   // Register Default Commands

   RegisterCommand( "quit",            "",    "",                      Console_Quit            );
   RegisterCommand( "backgroundcolor", "III", "<red> <green> <blue>",  Console_BackgroundColor );
   RegisterCommand( "about",           "",    "",                      Console_About           );
   RegisterCommand( "version",         "",    "",                      Console_Version         );
   RegisterCommand( "help",            "",    "",                      Console_Help            );
   RegisterCommand( "showallcommands", "",    "",                      Console_ShowAllCommands );
   RegisterCommand( "showfps",         "B",   "<true/false> or <1/0>", Console_ShowFps         );
}

/*************************************************************************************************/

void zConsole::ProcessCommand(zConsoleCommand &command)
{
   if (command.mpFunc == NULL) {
      _write("zConsole.ProcessCommand: mpFunc == NULL\n");
      return;
   }
   command.mpFunc(command);
}

/*************************************************************************************************/

void zConsole::ProcessMessage(zString message)
{
   zString info;

   message.TrimAll();
   message.Reduce(" ");
   int spacepos = message.Scan(' ');

   zString commandstr, paramstr;
   if (spacepos == -1) {      // Kein Space, d.h. keine Params
      commandstr = message;
      paramstr = "";
   }
   else {
      commandstr = message.SubL(spacepos);
      paramstr = message.SubR(message.Length()-spacepos-1);
   }

   for (int i=0; i < mpCommands.GetNum(); i++) {
      zConsoleCommand& command = mpCommands[i];

      // check command
      if (!strcmp(commandstr,command.GetCommandString())) {
         if (command.mpFunc == NULL) {
            AddLineToHistory(zString("This command has no execution function!"));
         }

         // check params
         if (GetNumParamsInString(paramstr) != command.GetNumParams()) {
            // falsch Anzahl von Params
            info.Format("error: %d params expected, but %d given!",command.GetNumParams(), GetNumParamsInString(paramstr));
            AddLineToHistory(info);

            // help String?
            if (command.GetHelpString().Length()) {
               AddLineToHistory(zString("Usage: ") + command.GetCommandString() + zString(" ") + command.GetHelpString());
               
               // display syntax
               zString syntaxstr = zString("Syntax: ") + command.GetCommandString() + zString(" ");
               for (int e=0; e < command.GetNumParams(); e++) {
                  switch (command.GetParamType(e)) {
                     case CON_PARAM_FLOAT:  syntaxstr += "<float> ";  break;
                     case CON_PARAM_INT:    syntaxstr += "<int> ";    break;
                     case CON_PARAM_STRING: syntaxstr += "<string> "; break;
                     case CON_PARAM_BOOL:   syntaxstr += "<bool> ";   break;
                     default:               syntaxstr += "<unknowntype> ";
                  }
               }
               syntaxstr.TrimAll();
               AddLineToHistory(syntaxstr);
            }
            return;
         }

         // num of params is correct
         zArray<zString>* paramArray = ConstructParams(paramstr);
         command.CreateVarsFromStrings(paramArray);
         ProcessCommand(command);
         delete paramArray;
         return;
      }
   }

   info.Format("Unknown Command '%s'...",commandstr);

   AddLineToHistory(info);
}

/*************************************************************************************************/

void zConsole::RegisterCommand(zConsoleCommand command)
{
   if (!command.HasValidParams()) {
      EM.WriteFormat("zConsole.RegisterCommand: %s hat invalide Params '%s'\n", command.GetCommandString(), command.GetParamString());
      return;
   }
   
   mpCommands.Add(command);
}

/*************************************************************************************************/

void zConsole::RegisterCommand(zString commandstr, zString paramcode, zString helpstr, void (*func)(zConsoleCommand&))
{
   zConsoleCommand command;

   command.SetCommandString(commandstr);
   command.SetParamString(paramcode);
   command.SetHelpString(helpstr);
   command.SetFunc(func);

   RegisterCommand(command);
}

/*************************************************************************************************/

void zConsole::RegisterCommand(zString totalstr, void (*func)(zConsoleCommand&))
{
   zString totalstr2 = totalstr;
   zString commandstr;

   totalstr.TrimAll();
   totalstr.Reduce(" ");
   int commapos = totalstr2.Scan(',');

   // Kein Comma -> keine Params
   if (commapos == -1) {
      totalstr2.TrimAll();
      RegisterCommand(totalstr2, "", "",func);
      return;
   }

   zString paramstr;
   commandstr = totalstr2.SubL(commapos);
   paramstr = totalstr2.SubR(totalstr2.Length()-commapos-1);
   commandstr.TrimAll();
   paramstr.TrimAll();

   commapos = paramstr.Scan(',');
   // Kein Comma -> Kein Help-String
   if (commapos == -1) {
      RegisterCommand(commandstr,paramstr,"",func);
      return;
   }

   zString helpstr;
   zString paramstr2;
   paramstr2 = paramstr.SubL(commapos);
   helpstr = paramstr.SubR(paramstr.Length()-commapos-1);
   paramstr2.TrimAll();
   helpstr.TrimAll();

   RegisterCommand(commandstr, paramstr2, helpstr, func);
}

/*************************************************************************************************/

bool zConsole::UnRegisterCommand(zString commandstr)
{
   // Alle Commands durchlaufen
   for (int i=0; i<GetNumCommands(); i++) {
       // Command gefunden -> unregister
      if (!strcmp(mpCommands[i].GetCommandString(),commandstr)) {
         mpCommands.Remove(i);
         return true;
      }
   }
   // Kein Command gefunden, unregister failed
   return false;
}

/*************************************************************************************************/

int zConsole::GetNumParamsInString(zString message)
{

   if (strlen(message) < 1) return 0;

   // Spaces lschen links und rechts

   int spacecount = 0;

   // Anzahl der Spaces zhlen "100 100 100" -> 2 Leerzeichen also 3 einzelne Strings
   for (int i=0; i < strlen(message); i++)
      if (message[i] == ' ')
         spacecount++;

   return spacecount+1;
}

/*************************************************************************************************/

zArray<zString>* zConsole::ConstructParams(zString paramstr)
{
   zArray<zString>* array = new zArray<zString>;
   zString checkstr = paramstr;

   checkstr.TrimAll();
   checkstr.Reduce(" ");

   int spacepos = checkstr.Scan(' ');

   zString newparam;
   zString checkstr2 = checkstr;

   // String zerlegen, solange bis kein Leerzeichen mehr existiert "100 100 100" -> drei einzelne
   while (spacepos != -1) {
      newparam = checkstr2.SubL(spacepos);
      array->Add(newparam);
      checkstr = checkstr2.SubR(checkstr2.Length()-spacepos-1);
      checkstr2 = checkstr;
      spacepos = checkstr2.Scan(' ');
   }
   
   if (checkstr.Length() > 0)
      array->Add(checkstr);

   return array;
}

/*************************************************************************************************/


/*************************************************************************************************/
/*************************************************************************************************/
// ***** public funcs

bool zConsole::Init(int width, int height, zString texfile, zString font, UINT fontheight,
                    DWORD colorkey, DWORD verdif)
{
   PreInit();
   mfWidth = width;
   mfHeight = height;
   mdwDiffuse = verdif & 0x00ffffff;
   mszUsedFont = new char[strlen(font)+1];
   mszUsedFont=font;
   miFontHeight = fontheight;

   _bttest(CreateStateBlocks(),
           "zConsole.Init: CreateStateBlocks failed\n");

   // Vertices setzen, Standard alle mit 0 ausser diffuse; Alpha auf 0
   verts[vlo].diffuse = mdwDiffuse;
   verts[vro].diffuse = mdwDiffuse;
   verts[vlu].diffuse = mdwDiffuse;
   verts[vru].diffuse = mdwDiffuse;
   verts[vro].x = mfWidth;
   verts[vro].tu = 1.0f;
   verts[vlu].tv = 1.0f;
   verts[vru].x = mfWidth;
   verts[vru].tu = 1.0f;
   verts[vru].tv = 1.0f;
   // y Koords werden durch Render() gesetzt

   // Font
   _bttest(mFont.Init(font.GetString(),fontheight),
           "zConsole.Init: Font.Init failed\n");
   _bttest(mFont.Restore(),
           "zConsole.Init: Font.Restore failed\n");

   InitCommands();

   return SetTexture(texfile, colorkey);
}

/*************************************************************************************************/

void zConsole::Shutdown()
{
   mFont.Invalidate();
   mFont.Shutdown();
   if (mdwStateBlock != 0xffffffff) pDev->DeleteStateBlock(mdwStateBlock);
   if (mdwSavedBlock != 0xffffffff) pDev->DeleteStateBlock(mdwSavedBlock);
   mszUsedFont="";
   mszTexfile="";
   _release(mpTexture);
   PreInit();
}

/*************************************************************************************************/

bool zConsole::Render()
{
   // Timing-Funktionen; Mssen bei jedem Rendergang ausgefhrt werden, auch wenn die
   // Console inaktiv ist;
   static DWORD oldtime = timeGetTime();
   DWORD time;
   float ftime;
   time = timeGetTime();
   ftime = 0.004f * (time - oldtime);
   oldtime = time;

   // Inaktive Console? Nichts zu rendern
   if (mState==CON_INACTIVE)
      return true;

   // Setze alle ntigen State, Texturen, Shader
   _httest(pDev->CaptureStateBlock(mdwSavedBlock),
           "zConsole.Render: CaptureStateBlock failed\n");
   _httest(pDev->ApplyStateBlock(mdwStateBlock),
           "zConsole.Render: ApplyStateBlock failed\n");
   _httest(_setvs(D3DFVF_CONSOLE),
           "zConsole.Render: _setvs failed\n");
   if (mbIsValidTexture) {
      _httest(pDev->SetTexture(0, mpTexture),
           "zConsole.Render: SetTexture failed\n");
   }

   // Console Position berechnen, wenn im ffnen/schlieen
   if (mState==CON_ACTIVATING) {
      verts[vlu].y += ftime * mfHeight;
      verts[vru].y += ftime * mfHeight;
      verts[vlu].y = _min(verts[vlu].y, mfHeight);
      verts[vru].y = _min(verts[vru].y, mfHeight);
      if (verts[vlu].y == mfHeight) mState = CON_ACTIVE;
   }
   else {
      if (mState==CON_DEACTIVATING) {
         verts[vlu].y -= ftime * mfHeight;
         verts[vru].y -= ftime * mfHeight;
         verts[vlu].y = _max(verts[vlu].y, 0.0f);
         verts[vru].y = _max(verts[vru].y, 0.0f);
         if (verts[vlu].y == 0.0f) mState = CON_INACTIVE;
      }
   }

   // Alpha fr Vertices berechnen
   if (mState==CON_ACTIVATING || mState==CON_DEACTIVATING) {
      DWORD alpha = (verts[vlu].y/mfHeight+50.0f) * 256;
      alpha = alpha << 24;
      verts[vlo].diffuse = mdwDiffuse | alpha;
      verts[vro].diffuse = mdwDiffuse | alpha;
      verts[vlu].diffuse = mdwDiffuse | alpha;
      verts[vru].diffuse = mdwDiffuse | alpha;
   }
   else {   
      // Voll geffnet? dann auch voll-sichtbare Verts
      if (mState==CON_ACTIVE) {
         verts[vlo].diffuse = mdwDiffuse | 0xff000000;
         verts[vro].diffuse = mdwDiffuse | 0xff000000;
         verts[vlu].diffuse = mdwDiffuse | 0xff000000;
         verts[vru].diffuse = mdwDiffuse | 0xff000000;
      }
   }

   // Console zeichnen
   _httest(_drawpu(D3DPT_TRIANGLESTRIP, 2, verts, sizeof(zconVertex)),
           "zConsole.Render: _drawpu failed\n");

   // Input-Zeile zeichnen
   if (strlen(mInput) > 0)
      _httest(mFont.DrawText(5,verts[vlu].y-miFontHeight-5,mdwFontLineColor,mInput),
              "zConsole.Render: DrawText failed\n");

   // History zeichnen
   int counter = miIndHistory-miHistoryOffset;
   for (int i=verts[vlu].y-(miFontHeight*2)-10; i > 0; i=i-miFontHeight-2) {
      // Gengen History Eintrge zum darstellen?
      --counter;
      if (counter < 0)
         if (miIndHistory < miNumHistoryEntries)
            counter = miNumHistoryEntries-1;

      mFont.DrawText(5,i,mdwFontHistoryColor,maHistory[counter]);
      if (miIndHistory-miHistoryOffset == counter)
         break;
   }

   // alten Zustand wieder herstellen
   _httest(pDev->ApplyStateBlock(mdwSavedBlock),
           "zConsole.Render: ApplyStateBlock failed\n");

   return true;
}

/*************************************************************************************************/

void zConsole::ChangeState()
{
   if (mState==CON_ACTIVE || mState==CON_ACTIVATING)
      mState = CON_DEACTIVATING;
   else
      mState = CON_ACTIVATING;
}

/*************************************************************************************************/

bool zConsole::SetTexture(zString texfile, DWORD colorkey)
{
   _release(mpTexture);
   mszTexfile="";
   mbIsValidTexture = false;
   // Soll eine Textur geladen werden?
   if (texfile != "") {
      mszTexfile = new char[strlen(texfile)+1];
      mszTexfile=texfile;
      _httest(D3DXCreateTextureFromFileEx(pDev, mszTexfile, 0, 0, 1, 0, D3DFMT_UNKNOWN,
                  D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, colorkey, NULL, NULL, &mpTexture),
              "zConsole.Init: D3DXCreateTextureFromFileEx failed\n");
      mbIsValidTexture = true;
   }
   return true;
}

/*************************************************************************************************/

void zConsole::Input(WPARAM input)
{

   // User drckt Enter
   if (input == VK_RETURN) {
      miHistoryOffset = 0;
      if (strlen(mInput) < 1)
         return;
      AddLineToCommandHistory();
      ProcessMessage(zString(mInput));
      if (mpFunc != NULL)
         mpFunc(mInput);
      for (int i=0; i < MAX_STR_LEN; i++) mInput[i] = '\0';
      miInputPos = -1;
      return;
   }

   // User drckt Back
   if (input == VK_BACK) {
      --miInputPos;
      if (miInputPos < -1)
         miInputPos = -1;
      mInput[miInputPos+1] = '\0';
      miShowCommand = miIndCommands-1;
      return;
   }

   // User drckt Space
   if (input == VK_SPACE) {
      ++miInputPos;
      if (miInputPos >= MAX_STR_LEN)
         miInputPos = MAX_STR_LEN-1;
      mInput[miInputPos] = ' ';
      miShowCommand = miIndCommands-1;
      return;
   }

   // User drckt A-Z oder 0-9
   if ((input > 64 && input < 91) || (input > 47 && input < 58)) {
      ++miInputPos;
      if (miInputPos >= MAX_STR_LEN)
         miInputPos = MAX_STR_LEN-1;
      mInput[miInputPos] = input;
      CharLowerBuff(mInput,miInputPos+1);
      miShowCommand = miIndCommands-1;
      return;
   }

   // User drckt Tab: Commandvervollstndigung
   if (input == VK_TAB) {
      TryCommandComplete();
      miShowCommand = miIndCommands-1;
      return;
   }

   // User drckt Pfeil hinauf
   if (input == VK_UP) {
      for (int i=0; i < MAX_STR_LEN; i++) mInput[i] = '\0';
      strcpy(mInput,maCommand[miShowCommand]);
      miInputPos = strlen(mInput)-1;
      --miShowCommand;
      if (miShowCommand < 0)
         miShowCommand = 0;
      return;
   }

   // User drckt Pfeil hinunter
   if (input == VK_DOWN) {
      for (int i=0; i < MAX_STR_LEN; i++) mInput[i] = '\0';
      ++miShowCommand;
      if (miShowCommand >= miNumCommands) {
         miShowCommand = miNumCommands;
         miInputPos = -1;
         return;
      }
      strcpy(mInput,maCommand[miShowCommand]);
      miInputPos = strlen(mInput)-1;
      return;
   }

   // Scrollen durch History
   int idiv = (verts[vlu].y-(miFontHeight*2)-10) / (miFontHeight+2);
   if (input == VK_LEFT) {
      miHistoryOffset += idiv;
      if (miHistoryOffset > miNumHistoryEntries-idiv)
         miHistoryOffset = miNumHistoryEntries-idiv;
      return;
   }
   if (input == VK_RIGHT) {
      miHistoryOffset -= idiv;
      if (miHistoryOffset < 0)
         miHistoryOffset = 0;
      return;
   }
}

/*************************************************************************************************/


