// ============================================================================================
// Datei:      zed4d.h
//
// Desc:       Header fr Zed4D.dll
//
// Notes:      ---
//             
// Coder:      Kongo
// Stand/Ver:  12.03.2002/0.8
// ============================================================================================

// Kompiliert mit DirectX SDK 8.1

#define ZED_D3D_3DMODEL_H
#define ZED_D3DBSP_H
#define ZED_D3D_PARTICLESYSTEM_H
#define ZED_D3D_INDEXANDVERTEXBUFFER_H
/*
   Nach einem halben Jahr freut es mich echt nicht mehr hier weiter zu programmieren. Es gibt
   einige Klassen die nicht wirklich ganz funktioniern, aber das ist mir jetzt egal. Fr 
   Anschauungszwecke reicht es noch, und einiges Material kann man doch verwenden. Also viel
   Spa damit. Sollte einmal etwas nicht funktionieren, fragt mich nicht warum. Fr andere
   Fragen bin ich aber immer offen.
   E-mail: kongo@gmx.at

                                                                            Kongo


WE DO NOT WARANTEE ANYTHING CONCERNING ANY OF THE FILES WHICH MAKE
UP THIS DLL. WE ACCEPT NO RESPONSIBILITY FOR ANY LOSS OR DAMAGE OF
ANY KIND, INCLUDING, BUT NOT LIMITED TO, LOSSES OF A PHYSICAL, 
MENTAL, SOCIAL, FINANCIAL, MARITAL, OR OF WHATEVER NATURE, RESULTING
FROM THE USE, OR THE PURPORTED USE, OF THIS DLL, FOR ANY PURPOSE 
WHATSOEVER. WE DO NOT EVEN WARANTEE THAT THE FILES WILL NOT KILL YOU.

YOU USE THIS DLL ENTIRELY AT YOUR OWN RISK, AND YOU SUPPLY
IT TO YOUR CUSTOMERS, FRIENDS, FAMILY, ACQUAINTANCES, OR ENEMIES,
ENTIRELY AT YOUR OWN RISK.

IF THESE TERMS ARE NOT ACCEPTABLE TO YOU, THEN PLEASE DELETE ALL
THE FILES FROM YOUR DISKS IMMEDIATELY AND PERMANENTLY.
  */




#ifndef ZED4D_IMPORT_H
#define ZED4D_IMPORT_H

#define ZED4D_VERSION   0.7

// Bei Kompilierung der Dll importieren, in einem Projekt exportieren
#ifdef ZED4D_EXPORTS
   #define ZAPI __declspec(dllexport)
#else
   #define ZAPI __declspec(dllimport)
#endif


// ============================================================================================
// Libs
// ============================================================================================

#include <windows.h>
#include <iostream.h>
#include <d3d8.h>
#include <d3dx8.h>
#include <dinput.h>
#include <dshow.h>
#include <vector>

// --------------------------------------------------------------------------------------------


class ZAPI zEngineManager
{
private:
	LPSTR			m_sLogFile;		// Log-Datei

   char *TranslateString (char *buf);

public:
	zEngineManager();
	~zEngineManager();

	/** LOGGING **/
	// Datei ndern
	void SetFile( LPCSTR, bool bDelete = false );
	// Dateinamen bergeben
	void GetFile( LPSTR ) const;
	// Log in Datei schreiben
	void Write( LPCSTR ) const;
   // Formatierte Log in Datei schreiben
   void WriteFormat( LPSTR, ...);
	// Fehlerbox + Trace
	void Error( LPCSTR ) const;
	void Error( LPCSTR, HRESULT ) const;

};
ZAPI zEngineManager EM;	// Enginemanager: Debug Ops, Trace,...

// ============================================================================================
// Definitionen ZIMouse
// ============================================================================================

// Definiert fr: IsDown, IsUp
#define ZIMOUSE_BUTTON_LEFT		0
#define ZIMOUSE_BUTTON_RIGHT		1
#define ZIMOUSE_BUTTON_MIDDLE		2
#define ZIMOUSE_BUTTON_EX1			3
#define ZIMOUSE_BUTTON_EX2			4
#define ZIMOUSE_BUTTON_EX3			5
#define ZIMOUSE_BUTTON_EX4			6
#define ZIMOUSE_BUTTON_EX5			7

// Definiert fr: GetPosChange(,)
#define ZIMOUSE_AXIS_X				0
#define ZIMOUSE_AXIS_Y				1
#define ZIMOUSE_AXIS_Z				2

// --------------------------------------------------------------------------------------------


// ============================================================================================
// Klasse ZInput
// ============================================================================================

class ZAPI zInput
{
private:
	// Das DirectInput Hauptobjekt
	static LPDIRECTINPUT8	m_pDI;

	// Anzahl der Create() Aufrufe
	static int					m_iCountRef;
public:

	// Konstruktor und Destruktor
	zInput();
	~zInput();

	// Startet/Beendet DirectInput
	bool Init(HINSTANCE);
	bool Shutdown();

	// Gibt das DirectInput Objekt zurck
	LPDIRECTINPUT8 GetInput();
};

// --------------------------------------------------------------------------------------------

// ============================================================================================
// Klasse ZKeyboardIm
// ============================================================================================

class ZAPI zKeyboard
{
private:

	// Device fr die Tastatur
	LPDIRECTINPUTDEVICE8		m_pdiDev;

	// Zhler fr Init()-Aufrufe
	int							m_iCountRef;

	// Zugriff auf DirectInput
	zInput						m_zInput;

	// Buffer fr Tastatur States
	char							m_KeyBuf[256];

	// Ist die Tastatur in unserer Anwendung aktiv
	bool							m_bActive;

	HRESULT						rval;
public:

	// Konstruktor, Destruktor
	zKeyboard();
	~zKeyboard();

	// Startet/Beendet den Zugriff auf die Tastatur
	bool Init(HINSTANCE, HWND);
	bool Shutdown();

	// Liest alle Keys
	bool Read();

	// berprft ob Key gedrckt wird
	bool IsDown(const int) const;

	// berprft ob Key nicht gedrckt wird
	bool IsUp(const int) const;

	// Aktiviert die Tastatur
	void Activate();

	// Deaktiviert die Tastatur
	void Deactivate();

	// Setzt die Tastatur aktiv oder inaktiv
	void SetMode(bool);
};

// --------------------------------------------------------------------------------------------

// ============================================================================================
// Klasse ZIMouseIm
// ============================================================================================

class ZAPI zMouse
{
private:

	// Device fr die Maus
	LPDIRECTINPUTDEVICE8		m_pdiDev;

	// Zhler fr Init()-Aufrufe
	int							m_iCountRef;

	// Zugriff auf DirectInput
	zInput						m_zInput;

	// Ist die Maus in der Anwendung aktiv
	bool							m_bActive;

	// Struktur fr Maus Infos
	DIMOUSESTATE2				m_diMouseState;

	HRESULT						rval;
public:
	
	// Konstruktor, Destruktor
	zMouse();
	~zMouse();

	// Initialisiert/Beendet die Maus
	bool Init(HINSTANCE, HWND);
	bool Shutdown();

	// Liest Infos der Maus
	bool Read();

	// Prft ob Maustaste gedrckt wird; mgliche Werte: ZIMOUSE_BUTTON_LEFT,...
	bool IsDown(const int) const;

	// Prft ob Maustaste nicht gedrckt wird; mgliche Werte: ZIMOUSE_BUTTON_LEFT,...
	bool IsUp(const int) const;

	// Speichert die Mausvernderungen in den Referenzen ab
	// Param1-3: x, y, z
	void GetPosChange(int&, int&, int&) const;

	// Speichert die Mausvernderung in der Variable ab
	// Param1: Definition; Param2: nderung der Axe
	void GetPosChange(const int, int&) const;

	// Aktiviert die Maus
	void Activate();

	// Deaktiviert die Maus
	void Deactivate();

	// Setzt den Mausmodus (aktiv oder nicht)
	// Param1: Mausmodus (aktiviert oder nicht)
	void SetMode(bool);

	// Informationen ber die Maus: Anzahl von Buttons und Axen
	// Param1: Anzahl von Buttons; Param2: Anzahl von Axen
	void GetCaps(int&, int&);

};

// --------------------------------------------------------------------------------------------


// ============================================================================================
// Klasse zVidPlayer
// ============================================================================================

class ZAPI zVidPlayer
{
private:
	// DirectShow Schnittstellen
	IGraphBuilder	*m_pGB;
	IMediaControl	*m_pMC;
	IMediaEventEx	*m_pME;
	IVideoWindow	*m_pVW;
	IMediaSeeking	*m_pMS;

	bool				m_bRender;

	HRESULT			rval;
	char*				err;
public:

	// Konstruktor und Destruktor
	zVidPlayer();
	~zVidPlayer();

	// Initialisiert den VideoPlayer
	// HWND - Wenn angegeben, ist das Videofenster ein Kindfenster; ansonsten erstellt der Player
	// ein eigenes (Fenster erst beim Abspielen sichtbar)
	// char* - Datei zum abspielen; untersttzte Formate siehe SDK
	bool Init(const char*, HWND);

	// Gibt den Videoplayer frei
	void Shutdown();

	/* Fenstereigenschaften */

	// Mgliche Flags sind dieselben wie die der Funktion CreateWindowEx, ausgenommen:
	// WS_DISABLED, WS_HSCROLL, WS_ICONIC, WS_MAXIMIZE, WS_MINIMIZE, WS_VSCROLL
	bool SetWindowStyleEx(long);
	// Mgliche Flags sind dieselben wie die der Funktion ShowWindow
	bool SetShowWindow( long);


	/* Videospielfunktionen */

	// Whrend des Abspielens bleibt die Kontrolle in der Funktion Render().
	// Diese ruft aber so oft wie mglich die angegeben Funktion auf. In der
	// angegebenen Funktion kann man dann weitere Kontrollfunktionen (Stop,...) aufrufen.
	// LPVOID - Zeiger auf Funktion, die whrend des Abspielens aufgerufen wird; bei Angabe von
	// NULL bearbeitet der Player alles selbst (Abbruch mit der Escape Taste erlaubt)
	// float - der Faktor um den das Videofenster vergrert werden soll (1 == 100% 2 == 200%)
	bool Render(void (*)(void), float);
	bool Play();
	bool Stop();
	bool Pause();
	bool Restart();

};

// --------------------------------------------------------------------------------------------


#ifndef ZED_VECTOR_H
#define ZED_VECTOR_H


// ============================================================================================
// Klasse zVector3
// ============================================================================================

class ZAPI zVector3: public D3DVECTOR
{
public:
   /* Konstruktoren */
   zVector3( void );
   zVector3( const float _x, const float _y, const float _z );
   zVector3( const zVector3& v0 );
   zVector3( float *p );

   void Set( const float _x, const float _y, const float _z);
   void Set( float *p );
   void Set( const zVector3& v0 );
   float Len( void );
   float Len_Squared( void );
   void Normalize( void );
   bool IsNormalized( void );
   bool IsEqualTo( const zVector3& v0, const float& fTol );
   void Inverse( void );
   void Inverse( zVector3 &v0 );
   float Distance( const zVector3& v0 );
   void Zero( void );
   float Angle( zVector3& v0 );
   void Minimum( const zVector3& v0 );
   void Maximum( const zVector3& v0 );

   /* Operatoren */
   zVector3&  operator  = ( const zVector3& v0 );
   zVector3&  operator  = ( const D3DVECTOR& v0 );
   const bool operator == ( const zVector3& v0 );
   const bool operator != ( const zVector3& v0 );
   zVector3&  operator += ( const zVector3& v0 );
   zVector3&  operator -= ( const zVector3& v0 );
   zVector3&  operator *= ( float s );
   zVector3&  operator /= ( float s );
   zVector3&  operator %= ( const zVector3& v1 );
   zVector3   operator  + ( const zVector3& v0 );
   zVector3   operator  - ( const zVector3& v0 );
   zVector3   operator  - ( void );
   zVector3   operator  * ( const float s );
   float      operator  * ( const zVector3& v0);
   zVector3   operator  / ( const float s );
   zVector3   operator  % ( const zVector3& v0 );
};

// --------------------------------------------------------------------------------------------


#endif

// ============================================================================================
// Klasse zMatrix44
// ============================================================================================

class ZAPI zMatrix44 : public D3DMATRIX
{
public:

	/* Konstruktor */
	inline zMatrix44( void );
	inline zMatrix44( float, float, float, float,
							float, float, float, float,
							float, float, float, float,
							float, float, float, float );
	inline zMatrix44( const zMatrix44& );
	inline zMatrix44( float * );

	/* Funktionen */
	inline void CopyTo( zMatrix44& ) const;
	inline void CopyTo( float * ) const;
	inline void Set( float, float, float, float,
						  float, float, float, float,
						  float, float, float, float,
						  float, float, float, float );
	inline void Set( const zMatrix44& );
	inline void Set( float * );
	inline void Ident( void );
	inline void Trans( void );
	inline float Det( void ) const;
	inline void Invert( void );

	inline void Mult( const zMatrix44& );
	inline void MultSimple( const zMatrix44& );

	inline void Translate( float, float, float );
	inline void Translate( const zVector3& );
	inline void Scale( float, float, float );
	inline void Scale( const zVector3& );
	inline void RotateX( float );
	inline void RotateY( float );
	inline void RotateZ( float );

	inline void LookAt(zVector3&, zVector3&);
	inline void SetViewMatrix( zVector3&, zVector3&, zVector3& );
	inline void SetProjMatrix( float, float, float, float );

	/* Operatoren */
	zMatrix44& operator *=( const zMatrix44& );
	zMatrix44& operator +=( const zMatrix44& );
	zMatrix44& operator -=( const zMatrix44& );
	zMatrix44& operator *=( const float );
	zMatrix44& operator /=( const float );
	zMatrix44& operator =( const zMatrix44& );
};

// --------------------------------------------------------------------------------------------

ZAPI inline zVector3 operator * ( const zMatrix44& , const zVector3& );
ZAPI inline zMatrix44 operator * ( const zMatrix44& , const zMatrix44& );
ZAPI inline zMatrix44 operator - ( const zMatrix44&, const zMatrix44& );
ZAPI inline zMatrix44 operator + ( const zMatrix44&, const zMatrix44& );
ZAPI inline zMatrix44 operator - ( const zMatrix44& );
ZAPI inline zMatrix44 operator / ( const zMatrix44&, const float );
ZAPI inline const bool operator == ( const zMatrix44&, const zMatrix44& );
ZAPI inline const bool operator != ( const zMatrix44&, const zMatrix44& );

// --------------------------------------------------------------------------------------------

// ============================================================================================
// Klasse zMiniTimer
// ============================================================================================

class ZAPI zMiniTimer
{
private:
	LONGLONG			m_llCurTime;			// die aktuelle Zeit
	LONGLONG			m_llLastTime;			// Zeit vom vorigen Aufruf 
	double			m_dTimeScale;			// Umwandeln in Mikrosekunden
	double			m_dTimeElapsed;		// Vergangene Zeit zwischen Aufrufen
	bool				m_bHighResCounter;	// Existiert der Counter
public:
	/* Konstruktor */
	zTimer( void );
	
	/* Funktionen */
	// Prft ob ein High-Resolution Counter zur Verfgung steht.
	bool Init( void );
	// Setzt alle Variablen wieder auf den Startwert zurck (wird noch nicht gebraucht)
	void Shutdown( void );
	// Gibt die Systemzeit in Sekunden zurck
	double GetTime( void );
	// Gibt die Zeit zurck seit dem letzten Aufruf von GetTime/GetElapsedTime;
	// Startzeit wird in der Funktion Init() gesetzt
	double GetElapsedTime( void );
};

// --------------------------------------------------------------------------------------------


// ============================================================================================
// Klasse zTimer
// ============================================================================================

class ZAPI zTimer
{
private:
	float m_fTicksPerSec;
	float m_fFramesPerSec;
	float m_fAverageFramesPerSec;
	float m_fSecsPerFrame;
	DWORD m_dwFrames;
	float m_fLastTime;
	float m_fInterval;
	float m_fLastInterval;

	UINT64 m_qwTicks;
	UINT64 m_qwStartTicks;
	UINT64 m_qwTicksPerSec;
	UINT64 m_qwTicksPerFrame;

public:
	zTimer();
	void Start(float fFramesPerSec);
	void Frame();
	UINT64 GetTicks();
	float GetDuration();
	float GetInterval() const { return m_fInterval; }
	float GetFramesPerSec() const { return m_fFramesPerSec; }
	float GetAverageFramesPerSec() const { return m_fAverageFramesPerSec; }
	float GetSecsPerFrame() const { return m_fSecsPerFrame; }
	UINT64 GetTicksPerSec() const { return m_qwTicksPerSec; }
	UINT64 GetTicksPerFrame() const { return m_qwTicksPerFrame; }
};

// --------------------------------------------------------------------------------------------


// ============================================================================================
// Klasse zProfTimer
// ============================================================================================

typedef struct zProfTimerInfo_tag {
friend class zProfTimer;
private:
	LONGLONG		llLastTime;				// Letzte Zeit des Aufrufes
	struct zProfTimerInfo_tag *next;	// Zeiger auf nchstes Item
	int			index;
public:
	double		dElapseTime;			// Zeit, nachdem die Funktion immer wieder aufgerufen werden soll
	void			(*pFunc)(void*);		// Funktion die aufgerufen wird
	void*			pData;					// Datenzeiger, der bergeben werden soll;
} zProfTimerInfo;

// --------------------------------------------------------------------------------------------

class ZAPI zProfTimer
{
private:
	LONGLONG				m_llTime;
	double				m_dTimeScale;
	bool					m_bHighResCounter;
	zProfTimerInfo*	m_head;
	int					m_iCount;

public:
	/* Konstruktor, Destruktor */
	zProfTimer( void );
	~zProfTimer( void );

	/* Funktionen */
	// Prft ob High Resolution Counter zur Verfgung steht
	bool Init( void );
	// Setzt alle Werte zurck
	void Shutdown( void );
	// berprft alle Timer, ob sie aktualisiert werden sollen
	void Update( void );
	// Erstellt einen Timer
	/* ElapseTime: Zeit nach der die Funktion immer wiedera aufgerufen wird
		pFunc:		Funktion die aufgerufen werden soll ( muss sein: void Func(void* Pointer); );
		pData:		Wird als Parameter der Funktion bergeben
	*/
	int AddTimer( double ElapseTime, void (*pFunc)(void*), void* pData );
	// Lscht den Timer mit dem Index iIndex
	// Achtung die Daten, auf die der pData Zeiger zeigt, werden nicht gelscht. Das muss der
	// Anwender selbst machen
	bool DeleteTimer( int iIndex );
	// Lscht alle Timer
	void DeleteAllTimer( void );
	// Findet den ersten Timer in der Liste, der auf den Speicher pData zeigt; -1 == existier nicht
	int FindTimer( void* pData );
	// Findet ersten Timer, der auf die Funktion pFunc zeigt
	int FindTimer( void (*pFunc)(void*) );
	// Findet ersten Timer, der ElapseTime hat
	int FindTimer( double ElapseTime );
};

// --------------------------------------------------------------------------------------------


#ifndef ZED_D3D_MANAGER_H
#define ZED_D3D_MANAGER_H


// ============================================================================================
// Defs
// ============================================================================================

//-----------------------------------------------------------------------------
// Name: struct zD3DInfo_Mode
// Desc: Structure for holding information about a display mode
//-----------------------------------------------------------------------------
struct zD3DInfo_Mode
{
    DWORD      Width;      // Screen width in this mode
    DWORD      Height;     // Screen height in this mode
    D3DFORMAT  Format;     // Pixel format in this mode
    DWORD      dwBehavior; // Hardware / Software / Mixed vertex processing
    D3DFORMAT  DepthStencilFormat; // Which depth/stencil format to use with this mode
};

//-----------------------------------------------------------------------------
// Name: struct zD3DInfo_Device
// Desc: Structure for holding information about a Direct3D device, including
//       a list of modes compatible with this device
//-----------------------------------------------------------------------------
struct zD3DInfo_Device
{
    // Device data
    D3DDEVTYPE       DeviceType;      // Reference, HAL, etc.
    D3DCAPS8         d3dCaps;         // Capabilities of this device
    const TCHAR*     strDesc;         // Name of this device

    // Modes for this device
    DWORD            dwNumModes;
    zD3DInfo_Mode    aModes[150];

    // Current state
    DWORD            dwCurrentMode;
    D3DMULTISAMPLE_TYPE MultiSampleType;
};

//-----------------------------------------------------------------------------
// Name: struct zD3DInfo_Adapter
// Desc: Structure for holding information about an adapter, including a list
//       of devices available on this adapter
//-----------------------------------------------------------------------------
struct zD3DInfo_Adapter
{
    // Adapter data
    D3DADAPTER_IDENTIFIER8 d3dAdapterIdentifier;

    // Devices for this adapter
    DWORD            dwNumDevices;
    zD3DInfo_Device  aDevices[5];

    // Current state
    DWORD            dwCurrentDevice;
};

// --------------------------------------------------------------------------------------------

struct zD3DInit {
   zD3DInfo_Adapter     Adapter[20];            // Enumerierte Grafikadapter
   DWORD                dwNumAdapters;           // Anzahl enumerierter Adapter
   DWORD                dwAdapter;              // Ausgewhlter Adapter
   DWORD                dwMinDepthBits;         // Z-Buffer Bits
   DWORD                dwMinStencilBits;       // Stencil Buffer Bits
};

// --------------------------------------------------------------------------------------------


// ============================================================================================
// Klasse: zD3DLight
// ============================================================================================

class ZAPI zD3DLight
{
private:
   D3DLIGHT8      Lights[8];
   bool           bLightActive[8];

   D3DMATERIAL8   Materials[8];
   bool           bMaterialActive[8];

   HRESULT        rval;
public:
   zD3DLight();
   ~zD3DLight();

   /* Lichter */
   bool Light_Enable(DWORD index);
   bool Light_Disable(DWORD index);
   bool Light_Set_Status(DWORD index, bool bEnable);
   bool Light_Set(DWORD index, D3DLIGHT8 *Light);
   bool Light_Get(DWORD index, D3DLIGHT8 *Light);
   bool Light_Set_Point(DWORD index, D3DCOLORVALUE diffuse, D3DCOLORVALUE specular,
                        D3DCOLORVALUE ambient, zVector3 vPos, float fRange, float fAtt1, 
                        float fAtt2, float fAtt3);
   bool Light_Set_Spot(DWORD index, D3DCOLORVALUE diffuse, D3DCOLORVALUE specular,
                       D3DCOLORVALUE ambient, zVector3 vPos, zVector3 vDir, float fRange, 
                       float fFallOff, float fAtt1, float fAtt2, float fAtt3, float fTheta, float fphi);
   bool Light_Set_Directional(DWORD index, D3DCOLORVALUE diffuse, D3DCOLORVALUE specular,
                              D3DCOLORVALUE ambient, zVector3 vDir);

   // nderungen werden direkt an D3D weiter gegeben.s
   bool Light_Set_Diffuse(DWORD index, D3DCOLORVALUE& diffuse);
   bool Light_Set_Specular(DWORD index, D3DCOLORVALUE& specular);
   bool Light_Set_Ambient(DWORD index, D3DCOLORVALUE& ambient);
   bool Light_Set_Pos(DWORD index, zVector3& pos);
   bool Light_Set_Dir(DWORD index, zVector3& dir);
   bool Light_Set_Range(DWORD index, float range);
   bool Light_Set_FallOff(DWORD index, float falloff);
   bool Light_Set_Att1(DWORD index, float att1);
   bool Light_Set_Att2(DWORD index, float att2);
   bool Light_Set_Att3(DWORD index, float att3);
   bool Light_Set_Theta(DWORD index, float theta);
   bool Light_Set_Phi(DWORD index, float phi);

   void Light_Get_Diffuse(DWORD index, D3DCOLORVALUE& diffuse);
   void Light_Get_Specular(DWORD index, D3DCOLORVALUE& specular);
   void Light_Get_Ambient(DWORD index, D3DCOLORVALUE& ambient);
   void Light_Get_Pos(DWORD index, zVector3& pos);
   void Light_Get_Dir(DWORD index, zVector3& dir);
   void Light_Get_Range(DWORD index, float& range);
   void Light_Get_FallOff(DWORD index, float& falloff);
   void Light_Get_Att1(DWORD index, float& att1);
   void Light_Get_Att2(DWORD index, float& att2);
   void Light_Get_Att3(DWORD index, float& att3);
   void Light_Get_Theta(DWORD index, float& theta);
   void Light_Get_Phi(DWORD index, float& phi);

   /* Alg. Lichtprops */
   bool Enable_Lighting();
   bool Disable_Lighting();
   bool Set_Lighting(bool bLight);
   bool Get_Lighting();

   bool Set_AmbientColor(D3DCOLOR clrAmb);
   D3DCOLOR Get_AmbientColor();

   bool Enable_Specular();
   bool Disable_Specular();
   bool Set_Specular(bool bSpec);
   bool Get_Specular();

   bool Set_Shading_Flat();
   bool Set_Shading_Gouraud();
   bool Set_Shading(DWORD dwShadeMode); //D3DSHADE_FLAT, D3DSHADE_GOURAUD
   DWORD Get_Shading();

   /* Materials */
   // Material Sets werden nicht automatisch an D3D weitergegeben
   void Material_Set(DWORD index, D3DMATERIAL8& mat);
   void Material_Set(DWORD index, D3DCOLORVALUE& diffuse, D3DCOLORVALUE& ambient,
                     D3DCOLORVALUE& specular, D3DCOLORVALUE& emissive, float Power);

   void Material_Set_Diffuse(DWORD index, D3DCOLORVALUE& diffuse);
   void Material_Set_Ambient(DWORD index, D3DCOLORVALUE& ambient);
   void Material_Set_Specular(DWORD index, D3DCOLORVALUE& specular);
   void Material_Set_Emissive(DWORD index, D3DCOLORVALUE& emissive);
   void Material_Set_Power(DWORD index, float Power);

   void Material_Get(DWORD index, D3DMATERIAL8& mat);
   void Material_Get_Diffuse(DWORD index, D3DCOLORVALUE& diffuse);
   void Material_Get_Ambient(DWORD index, D3DCOLORVALUE& ambient);
   void Material_Get_Specular(DWORD index, D3DCOLORVALUE& specular);
   void Material_Get_Emissive(DWORD index, D3DCOLORVALUE& emissive);
   void Material_Get_Power(DWORD index, float& Power);

   bool Material_Activate(DWORD index);
   bool Material_Activate(DWORD index, D3DMATERIAL8& mat);

   /* Fog */
   bool Fog_Enable();
   bool Fog_Disable();
   bool Fog_Set_Status(bool bEnable);
   bool Fog_Start(float fStart);
   bool Fog_End(float fEnd);
   bool Fog_Density(float fDensity);
   bool Fog_Mode(DWORD dwFogMode);
   bool Fog_Color(DWORD dwColor);
   // dwFogMode = D3DFOG_NONE,D3DFOG_EXP(fDensity),D3DFOG_EXP2(fDensity),D3DFOG_LINEAR(fStart,fEnd)
   bool Fog_Set(DWORD dwFogMode=D3DFOG_LINEAR,float fDensity=0.0005f,float fStart=0.0f,float fEnd=1.0f,BYTE r=0,BYTE g=0,BYTE b=0,BYTE a=255,bool bEnabled=true);
};

// --------------------------------------------------------------------------------------------


// ============================================================================================
// Klasse: 
// ============================================================================================

// --------------------------------------------------------------------------------------------


// ============================================================================================
// Klasse: zTextureManager
// ============================================================================================

class ZAPI zD3DManager
{
protected:

   // Struktur mit den Initalisierungswerten
   zD3DInit          *m_psInit;
   // Initialisierungsstruktur fr Direct3D
   D3DPRESENT_PARAMETERS     m_d3dpp;
   // Mglichen Modes bei einem Reset
   zD3DInfo_Device   *m_psResetDevice;
   // Resestfunktionen
   bool (*m_pDestroyFunc)(void);
   bool (*m_pCreateFunc)(void);
   bool (*m_EnumDeviceOkay)(D3DCAPS8* pddDriverCaps, DWORD dw, D3DFORMAT d3dfrmt);


   HRESULT           rval;
   // Fr ScreenShot()
   char* GetFileName(char* szFile);

   /* Funktionen zum initialisieren eines Devices */
   // Enumeriert alle passenden Devices und deren Modes
   bool EnumDeviceList(DWORD dwWidth,        DWORD dwHeight,
                       DWORD dwBpp,          DWORD dwZBits,
                       DWORD dwStencilBits,  bool bInitManual,
                       HWND  hWnd,           HINSTANCE hInst);
   // Sucht uns ein passendes Depth/Stencil Format
   bool FindDepthStencilFormat(UINT        iAdapter,
                               D3DDEVTYPE  DeviceType,
                               D3DFORMAT   TargetFormat,
                               D3DFORMAT*  pDepthStencilFormat,
                               DWORD       dwMinDepthBits,
                               DWORD       dwMinStencilBits);
   // Erstellt uns den Device
   bool CreateEnumDevice(HWND hWnd, bool bSync);

public:
   /* Klassenvars */

   zD3DLight      Light;


   /* Funktionen */

   zD3DManager();
   ~zD3DManager() { Shutdown(); }

   // Initialisiert einen Device
   bool Init(HWND hWnd,            HINSTANCE hInst,
             DWORD dwWidth,        WORD dwHeight,
             DWORD dwBpp,          DWORD dwZBits,
             DWORD dwStencilBits,  /*DWORD dwTryFormat,*/
             bool bSync,           bool bInitManual,
             bool (*EnumDeviceOkay)(D3DCAPS8* pddDriverCaps, DWORD dw, D3DFORMAT d3dfrmt));
   // Gibt alles frei
   bool Shutdown();
   // Prfung auf Reset false == Reset ntig
   bool CheckReset();
   // Reset mit neuen Werten
   bool Reset(DWORD dwWidth=0,       DWORD dwHeight=0,
              DWORD dwBpp=0,         bool bSync=false);
   // Vor einem Reset wird die Funktion DestroyFunc und nach einem Reset die Funktion
   // CreateFunc aufgerufen. Dient dazu, um Ressourcen zu zerstren/initalisieren
   void SetResetFuncs(bool (*pDestroyFunc)(void), bool (*pCreateFunc)(void));
   // Macht einen ScreenShot des Frontbuffers. Wenn kein Dateiname angegeben wird, wird die
   // Datie austomatisch unter 'screen shot\shotxxx.bmp" abgespeichert.
   bool ScreenShot(LPCSTR szFile = NULL);
};

// --------------------------------------------------------------------------------------------


#endif

ZAPI IDirect3D8*          pD3D;
ZAPI IDirect3DDevice8*    pDev;

// --------------------------------------------------------------------------------------------

class ZAPI zD3DCamera
{
private:

public:
   // Variablen fr die Ausrichtung
   float     fYaw,
             fPitch,
             fRoll;
   zVector3  vUp,
             vDir,
             vRight;
   zVector3  vPos;

   zD3DCamera();
   ~zD3DCamera() {};

   zVector3 GetPos();
   void SetPos(zVector3 *vNewPos);

   // Kamera verschieben, rotieren
   void Activate(float _x, float _y, float _z);
   void Translate(float _x, float _y, float _z);
   void Rotate(float _x, float _y, float _z);
   // Matrix erstellen und Direct3D bergeben
   bool Update();
};


// --------------------------------------------------------------------------------------------




#ifndef ZED_D3DTEXTUREMANAGER_H
#define ZED_D3DTEXTUREMANAGER_H


// ============================================================================================
// Klasse: zTextureManager
// ============================================================================================

#define zMAXSIZE_TEXTURE_STRING      50    // Maximale Lnge eines Dateinamen
#define zMAX_TEXTURESTAGES            8    // Anzahl der Texturstages
typedef WORD zTexture_ID;                  // Texture_ID 
#define zINVALID_TEXTURE         0xFFFF

struct zTextureNode {
   int                  numReferences;
   LPDIRECT3DTEXTURE8   pTexture;
   char                 szName[zMAXSIZE_TEXTURE_STRING];
};


class ZAPI zD3DTextureManager
{
private:
   int m_NumTextures;
   int m_MaxTextures;
   zTextureNode **m_TextureArray;
   HRESULT rval;

   void ExpandMemoryBuffer(int newSize);
   // Fr ScreenShot(); berprft ob Dateien schon existieren
   char* GetFileName(char* szFile);

public:
   zD3DTextureManager();
   ~zD3DTextureManager();

   // Ladet Texture von Datei
   zTexture_ID LoadFromFile(LPCSTR szFile);

   // Bindet eine Textur in einen TextureStage
   bool BindTexture(zTexture_ID id, int stage = 0);
   // Setzt den TextureStage auf NULL
   void UnbindTexture(int stage = 0);
   // Gibt die Textur frei
   bool RemoveTexture(zTexture_ID id);
   // Gibt den Texturdateinamen zurck
   char* GetTextureName(char* szName, zTexture_ID id);
   // Anzahl der Texturen im Array
   int GetNumTextures();

   // Gibt alle Texturen frei, behlt aber die Dateinamen. Dies ist fr einen Reset gedacht.
   void ReleaseTextureHandles();
   // Ladet alle Texturen wieder nach einem Reset.
   void RestoreTextureHandles();

   // Texturmodsfuncs

   // Sperrt einen bestimmten Bereich der Textur. Dies ist nur eine Kapselung von 
   // IDirect3DTexture8::LockRect
   bool Texture_Lock(zTexture_ID id, UINT Level, D3DLOCKED_RECT *pLockedRect, const RECT *pRect, DWORD Flags);
   void Texture_Unlock(zTexture_ID id, UINT Level);

   // Ladet ein Bild(Alpha) in den Alphabereich der Textur
   bool Texture_LoadAlphaInto(char *szFile, zTexture_ID id);
   
   // ACHTUNG!! SetAlphaFuncs funktionieren noch nicht

   // Alle Pixel der Textur mit der Farbe (red,green,blue) bekommen den Alphawert alpha
   void Texture_SetAlphaColor(zTexture_ID id, BYTE red, BYTE green, BYTE blue, BYTE alpha);
   // Jeder Pixel bekommt den Alphawert alpha
   void Texture_SetAlphaSurface(zTexture_ID id, BYTE alpha);
   // x,y,x2,y2 == Rect; Alle Pixel in diesem Rect bekommen den Alphawert alpha
   void Texture_SetAlphaRegion(zTexture_ID id, WORD x, WORD y, WORD x2, WORD y2, BYTE alpha);
};

// --------------------------------------------------------------------------------------------


#endif


#ifndef ZED_D3DBSP_H
#define ZED_D3DBSP_H


// ============================================================================================
// Definitionen
// ============================================================================================

// ViewFrustrum Seiten: 
#define zVF_L 0    // Linke Clipping Plane
#define zVF_R 1    // Rechte Clipping Plane
#define zVF_T 2    // Obere Clipping Plane
#define zVF_B 3    // Untere Clipping Plane
#define zVF_N 4    // Nahe Clipping Plane
#define zVF_F 5    // Ferne Clipping Plane

// Culling Ergebnisse:
#define zNWERT int
#define zBOX_INSIDE  0
#define zBOX_OUTSIDE 1
#define zBOX_CLIPPED  2

/**
 * Struktur fr eine Ebene. Diese kann man mathematisch durch
 * ihren Normalenvektor (senkrecht zur Ebene) und die Entfernung
 * zum Nullpunkt (0,0,0) beschreiben.
 */ 
typedef struct ZAPI zPlane_tag {
   zVector3  vNormal;   // Normalenvektor der Ebene
   float     fDist;     // Distanz zum Nullpunkt
} zPlane;

/**
 * Grsste und kleinste Ecke einer Box fr das Culling
 */
typedef struct ZAPI zBox_tag {
   zVector3  vMin;         // Kleinster Eckpunkt der Box
   zVector3  vMax;         // Grsster Eckpunkt der Box
   zVector3  vCenter;      // Mittelpunkt (Zentrum) der Box
} zBox;

// --------------------------------------------------------------------------------------------

// Maximale Baumgrsse
#define zBSP_MAX_NODES  10000

// Polygon Klassifizierungen
#define zBSP_FRONT    0
#define zBSP_BACK     1
#define zBSP_PLANAR   2
#define zBSP_SPANNING 3

// Rechengenauigkeit:
#define zBSP_DELTA 0.00001

// Fr die Polygone:
#define zBSP_POLY_MAX_VERTICES  20
#define zBSP_POLY_MAX_INDICES   40
#define zBSP_POLY_MAX_TEXTURES 256

// --------------------------------------------------------------------------------------------

typedef struct ZAPI zVertex_Bsp_tag {
   float x,y,z;
   DWORD diffuse;
   float tu,tv;
} zVertex_Bsp;

#define D3DFVF_BSP (D3DFVF_XYZ|D3DFVF_DIFFUSE|D3DFVF_TEX1)

// --------------------------------------------------------------------------------------------

// Struktur fr ein Polygon
typedef struct ZAPI zPoly_tag {
   zVertex_Bsp  vVerts[zBSP_POLY_MAX_VERTICES];  // Vertices
   long         lNumVerts;                       // Anzahl der Vertices
   WORD         wIndices[zBSP_POLY_MAX_INDICES]; // Indices auf Vertices
   long         lNumIndices;                     // Anzahl der Indices
   zPlane       Plane;                           // Ebene des Polys
   bool         bSplitter;                       // Schon mal Splitter gewesen?
   zTexture_ID  tex_id;                          // Textur ID des Polys
} zBspPoly;

// --------------------------------------------------------------------------------------------

// Struktur fr Root, Nodes und Leafs des BSP Baumes
typedef struct ZAPI zNode_tag {
   zPlane       Plane;      // NODE: Teilungsebene
   UCHAR        bLeaf;      // NODE oder LEAF
   long         lNumPolys;  // Anzahl der Polygone
   zBspPoly    *pPolys;     // LEAF: Liste der Polygone
   zNode_tag   *pFront;     // NODE: Frontliste
   zNode_tag   *pBack;      // NODE: Backliste
   zBox         Box;        // Bounding Box
} zBspNode;

// --------------------------------------------------------------------------------------------

// Defs fr Render_Special
#ifdef _DEBUG
#define BSPR_WIREFRAME        0x00000001L
#define BSPR_ALL              0x00000002L
#define BSPR_NOTHING          0x00000004L
                              //0x00000008L
#define BSPR_BOXES            0x00000010L
#define BSPR_BOXES_ALL        0x00000020L
#define BSPR_BOXES_ONLY       0x00000040L
                              //0x00000080L
#define BSPR_CULL_NONE        0x00000100L
#define BSPR_CULL_CCW         0x00000200L
#define BSPR_CULL_CW          0x00000400L
                              //0x00000800L
#define BSPR_NOTEXTURES       0x00001000L
//0x00002000L
//0x00004000L
//0x00008000L

#define BSPR_LOOK_GOD         0x00010000L
#define BSPR_LOOK_PLUS        0x00020000L
#define BSPR_LOOK_MINUS       0x00040000L
//0x00080000L
#endif


// --------------------------------------------------------------------------------------------


// ============================================================================================
// Klasse: zD3DBsp
// ============================================================================================

class ZAPI zD3DBsp {
public:
   zD3DBsp();
   ~zD3DBsp();
   bool Init(char *szPath, char *szFile);       // PHASE 1: Spiel starten
   bool Render(zVector3 &vPos);    // PHASE 2: Hauptschleife
   void Shutdown(void);           // PHASE 3: Spiel beenden

#ifdef _DEBUG
   bool Render_Special(zVector3 &vPos, DWORD Flags); // Spezielle Renderfunktionen
   zVector3  m_vPos;
   float     m_fDist;
   DWORD m_dwRenderFlags;
       // Daten immediate
   int m_iLeafs_rendered,
       m_iTris_rendered,
       m_iVerts_rendered,
       m_iInds_rendered,
       m_iPolys_rendered,
       // Daten statisch: Es werden aber keine Daten bei Split_Poly bernommen, d.h.
       // bei Teilung eines Polygon, Daten werden nicht updated
       m_iNumLeafs,
       m_iNumTris,
       m_iNumVerts,
       m_iNumInds,
       m_iNumBoxes,
       m_iNumPolys;

   // Hier ein paar Variablen fr Testausgaben ins
   // Protokoll, nur des Interesses halber!!!
   DWORD dwNumSplits,    // Wie viele Splits verursachen wir?
         dwNumNodes;     // Wieviel Elemente hat Baum zur Wurzel?
#endif

protected:

   // Wurzel des Trees
   zBspNode           m_Node0;
   // Textureholder
   zD3DTextureManager m_TexMan;

   // Tree Vars
   zBspNode m_aNode_Pool[zBSP_MAX_NODES];
   long  m_lPool_Index;
   // View Frustrum Ebenen
   zPlane             m_VFrustum[6];

   HRESULT            rval;

   // Member Funktionen:
   // ==================
   bool IntersectionPoint(zVector3 *linestart,    zVector3 *lineend,
                          zVector3 *vertex,       zVector3 *normal,
                          zVector3 *intersection, float    *percentage);
   void Create_ViewFrustum();
   zNWERT Cull_AABB(zBox *pAABB);

   // Laden der Leveldaten aus Datei
   bool  LoadLevelData(LPSTR szPath, LPSTR szFile);

   // Rekursive Erstellung des Baumes:
   void Create_BSP_Tree(zBspNode *pNode);
   zBox Create_BoundingBox(zBspPoly *pPolylist, long lNumPolys);
   bool Find_Splitter(zBspPoly *pPolylist, long lNumPolys, zPlane *pSplitter);

   // Rendern des fertigen Baumes
   void Render_recursive(zBspNode *pNode, zVector3 &vPos);
   void Render_Leaf(zBspNode *pNode);

   // Hilfsfunktionen fr 3D Mathemathik
   int Classify_Point(zPlane Plane, zVector3 vPoint);
   int Classify_Poly(zPlane Plane, zBspPoly *pPoly);
   zPlane PolyToPlane(zBspPoly *pPoly);
   void Split_Poly(zBspPoly *pPoly,       zPlane *pPlane,
                   zBspPoly *pFrontSplit, zBspPoly *pBackSplit);

   // BSP Baum Speicher freigeben
   void Kill_recursive(zBspNode *pNode);
};

// --------------------------------------------------------------------------------------------


#endif


#ifndef ZED_D3D_FONT_H
#define ZED_D3D_FONT_H


// ============================================================================================
// Defs
// ============================================================================================

#define zFONT_MAX_VERTICES 50*6

typedef struct zVertex_Font2D_tag {
   float x,y,z;
   float rhw;
   DWORD diffuse;
   float tu,tv;
} zVertex_Font2D;

#define D3DFVF_FONT2D (D3DFVF_XYZRHW|D3DFVF_DIFFUSE|D3DFVF_TEX1)

inline zVertex_Font2D zInitVertex_Font2D( const zVector3& p, float rhw, D3DCOLOR color, float tu, float tv)
{
   zVertex_Font2D v;
   v.x = p.x; v.y = p.y; v.z = p.z; v.rhw = rhw;
   v.diffuse = color; v.tu = tu; v.tv = tv;
   return v;
}

// --------------------------------------------------------------------------------------------

typedef struct zVertex_Font3D_tag {
   float x,y,z;
   D3DVECTOR n;
   float tu,tv;
}zVertex_Font3D;

#define D3DFVF_FONT3D (D3DFVF_XYZ|D3DFVF_NORMAL|D3DFVF_TEX1)

inline zVertex_Font3D zInitVertex_Font3D(const zVector3& p, const zVector3& n, float tu, float tv )
{
    zVertex_Font3D v;
    v.x = p.x;   v.y = p.y;   v.z = p.z;
    v.n.x = n.x; v.n.y = n.y; v.n.z = n.z;
    v.tu = tu;   v.tv = tv;
    return v;
}


// --------------------------------------------------------------------------------------------

// Font creation flags
#define zFONT_BOLD        0x0001
#define zFONT_ITALIC      0x0002

// Font rendering flags
#define zFONT_CENTERED    0x0001
#define zFONT_TWOSIDED    0x0002
#define zFONT_FILTERED    0x0004

// --------------------------------------------------------------------------------------------


// ============================================================================================
// Klasse
// ============================================================================================

class ZAPI zD3DFont
{
private:
   TCHAR          m_strFontName[80];         // Fontprops
   DWORD          m_dwFontFlags,
                  m_dwFontHeight;

   IDirect3DVertexBuffer8 *m_pVB;

   IDirect3DTexture8      *m_pTex;           // Textur fr die Font
   DWORD          m_dwTexWidth,
                  m_dwTexHeight;
   float          m_fTextScale,
                  m_fTexCoords[128-32][4];
   
   // Stateblocks for setting and restoring render states
   DWORD   m_dwSavedStateBlock;
   DWORD   m_dwDrawTextStateBlock;

   HRESULT        rval;

public:
   zD3DFont();
   ~zD3DFont();

   bool Init(TCHAR *strFontName, DWORD dwHeight, DWORD dwFlags=0L);
   bool Restore();
   bool Invalidate();
   bool Shutdown();

    // 2D and 3D text drawing functions
   bool DrawText(float sx, float sy, DWORD dwColor,
                 TCHAR* strText, DWORD dwFlags=0L);
   bool DrawText(float x, float y, float z,
                 float fXScale, float fYScale, DWORD dwColor,
                 TCHAR* strText, DWORD dwFlags=0L);
   bool Draw3DText(TCHAR* strText, DWORD dwFlags=0L);
   // Function to get extent of text
   bool GetTextDimensions(TCHAR* strText, SIZE* pSize);
};

// --------------------------------------------------------------------------------------------


#endif

// --------------------------------------------------------------------------------------------



// --------------------------------------------------------------------------------------------


#ifndef ZED_D3D_INDEXANDVERTEXBUFFER_H
#define ZED_D3D_INDEXANDVERTEXBUFFER_H


// ============================================================================================
// Klassen
// ============================================================================================

// VertexBuffer
class ZAPI zD3DVertexBuffer
{
private:
   UINT           m_uiVertsInBuffer;
   UINT           m_uiBindStreamNumber;
   UINT           m_uiNumVertices;
   UINT           m_uiVertexSize;
   DWORD          m_dwFVF;
   IDirect3DVertexBuffer8  *m_pVB;

   HRESULT        rval;
public:
   zD3DVertexBuffer();
   ~zD3DVertexBuffer();

   bool Init(UINT uiVertexSize, UINT uiNumVertices, DWORD dwUsage, DWORD dwFVF,
             D3DPOOL Pool, UINT SizeToLock, BYTE** ppData, DWORD Flags=NULL);
   bool Shutdown();

   bool CopyVertices(UINT uiOffLock, UINT uiSizeLock, BYTE** ppData, DWORD Flags=NULL);
   bool SetShader();
   bool BindStream(UINT uiStreamNumber);
   bool UnbindStream();

   UINT GetNumVertices();
   UINT GetVertexSize();
   UINT FreeVertices();
   DWORD GetFVF();
   UINT GetBufferSize();
};

// --------------------------------------------------------------------------------------------

// IndexBuffer
class ZAPI zD3DIndexBuffer
{
private:

   D3DFORMAT      m_IndFormat;
   UINT           m_uiMaxIndices;
   UINT           m_uiIndsInBuffer;
   IDirect3DIndexBuffer8   *m_pIB;

   HRESULT       rval;
public:
   zD3DIndexBuffer();
   ~zD3DIndexBuffer();

   bool Init(UINT Len, DWORD dwUsage, D3DFORMAT Format, D3DPOOL Pool, UINT SizeToLock, BYTE** ppData, DWORD Flags=NULL);
   bool Shutdown();
   bool CopyIndices(UINT uiOffLock, UINT uiSizeLock, BYTE** ppData, DWORD Flags=NULL);
   bool SetIndices(UINT BaseVIndex);

   UINT GetNumIndices();
   UINT GetNumFreeVertices();
};


#endif
// --------------------------------------------------------------------------------------------


#ifndef ZED_D3D_PARTICLESYSTEM_H
#define ZED_D3D_PARTICLESYSTEM_H


// ============================================================================================
// Defs
// ============================================================================================

// Partikel
struct ZAPI zParticle
{
   zVector3   vPos;
   zVector3   vOldPos;
   zVector3   vVel;
   zVector3   vForce;

   float      fLifetime;
   float      fAge;

   D3DCOLORVALUE  clrCur;
   D3DCOLORVALUE  clrCur_delta;

   float      fSize;
   float      fSize_delta;

   // Wenn fAge > fLifetime Rckgabe == false. Partikel wir automatisch von Update() als
   // tot gekennzeichnet, durch fAge = -1.0f.
   bool Update(float time_delta);
}; 

// --------------------------------------------------------------------------------------------

// Partikel Vertex
struct zVertex_Particle
{
   zVector3  v;
   float     psize;
   D3DCOLOR  color;
};

#define D3DFVF_PARTICLE (D3DFVF_XYZ|D3DFVF_PSIZE|D3DFVF_DIFFUSE)

// --------------------------------------------------------------------------------------------

#define RAND3 (rand()%5)

// --------------------------------------------------------------------------------------------


// ============================================================================================
// Klasse zD3DParticleSystem
// ============================================================================================

class ZAPI zD3DParticleSystem
{
private:
  /* Systemidentification */
   // Identifikationsnummer des Systems
   int            m_ID;
   // Name des Systems
   char           m_szName[64];
   // System-Position
   zVector3       m_vSystemPos;
   // Bewegungs-Vektor des Systems
   zVector3       m_vSystemVel;
   // Systembewegung?
   bool           m_bSystemIsMoving;
   // Alter in Sekunden; -1.0f == Nicht mehr gltig; System wurdem mit Shutdown beendet.
   // Wird nur bei Init() mit timeGetTime() gesetzt. Alter = 0.001f*(timeGetTime()-Age)
   float          m_fSystemAge;
   
  /* D3D Vars */
   IDirect3DVertexBuffer8*  m_pVB;
   HRESULT        rval;

  /* Partikel-Array (allokiert whrend Init() */
   DWORD          m_dwMaxParticles;
   zParticle      *m_pParticles;

  /* Particlegeneration */
   
   // Partikel pro Sekunde emittieren
   float          m_fParEmitPerSec;
   // Anzahl der aktiven Partikel
   DWORD          m_dwNumParticles;
   // Keine neuen Partikel. Neue Partikel speichern in EmissionResidue. Damit sind Explosionen
   // mglich.
   bool           m_bParSupress;
   // Keine neuen Partikel. Diese aber auch nicht in EmissionResidue speichern.
   bool           m_bNoNewPar;
   // Letzte Updatezeit in Sekunden
   float          m_fTimeLastUpdate;
   // Wenn bei Updateversuch 0.4 Partikel erzeugt werden sollen, dann wird die Kommazahl fr
   // den nchsten Updategang aufgehoben.
   float          m_fEmissionResidue;

  /* Particle-Init */

   // Der Start-Wert wird immer mit dem Var-Wert kombiniert. Dabei kann der Var-Wert bis zu
   // 3x in den Start-Wert einflieen. Also: Init = Start + Var * (0-3); Der End-Wert ist immer
   // der gleiche. 

   // Positions-Variation
   zVector3       m_parvPosVar;
   // Beschleunigungsvektor
   zVector3       m_parvVel;
   zVector3       m_parvVelVar;
   // Kraftwirkung auf Vel-Vektor
   zVector3       m_parvForce;
   zVector3       m_parvForceVar;
   // Punktgre
   float          m_parfSizeStart;
   float          m_parfSizeEnd;
   float          m_parfSizeVar;
   // Partikelfarbe
   D3DCOLORVALUE  m_parclrStart;
   D3DCOLORVALUE  m_parclrEnd;
   D3DCOLORVALUE  m_parclrVar;
   // Lebensdauer
   float          m_parfLifetime;
   float          m_parfLifetimeVar;

public:
   zD3DParticleSystem();
   ~zD3DParticleSystem();

   // Initialisiert die System-Vars (ausser Par-Generation und Par-Init-Werte) und erstellt
   // den VertexBuffer fr maximal dwMaxParticles
   bool Init(char* szName, int id, zVector3 vSystemPos, zVector3 vSystemVel=zVector3(0.0f,0.0f,0.0f),
             bool bSystemMoving=false, DWORD dwMaxParticles=256);

   bool Shutdown();

   // Aktualisiert das System (Partikel); Parameter = Systemzeit in Sec (0.001f*timeGetTime())
   // Wenn keine Partikel mehr existieren, gibt Update() false zurck.
   bool Update(float time);
   // Render das ganze Partikelsystem
   bool Render();

  /* pre-defined Props */
   bool LoadPropsFromFile(char *szFile);
   bool SavePropsToFile(char *szFile);

  /* Set-Funcs */
   void Set_System_ID(int id);
   void Set_System_Name(char *szName);
   void Set_System_Pos(zVector3 vPos);
   void Set_System_Vel(zVector3 vVel);
   void Set_System_Moving(bool bMoving);
   
   void Set_Emitter_EmitPerSec(float fEmit);
   void Set_Emitter_Supress(bool bSupress);
   void Set_Emitter_Inactive(bool bNewPar);

   void Set_Par_PosVar(zVector3 vPosVar);
   void Set_Par_Vel(zVector3 vVelStart);
   void Set_Par_VelVar(zVector3 vVelVar);
   void Set_Par_Force(zVector3 vForceStart);
   void Set_Par_ForceVar(zVector3 vForceVar);
   void Set_Par_SizeStart(float fSizeStart);
   void Set_Par_SizeEnd(float fSizeEnd);
   void Set_Par_SizeVar(float fSizeVar);
   void Set_Par_ColorStart(D3DCOLORVALUE clrStart);
   void Set_Par_ColorEnd(D3DCOLORVALUE clrEnd);
   void Set_Par_ColorVar(D3DCOLORVALUE clrVar);
   void Set_Par_Lifetime(float fLifetime);
   void Set_Par_LifetimeVar(float fLifetimeVar);

  /* Get-Funcs */
   void Get_System_ID(int &id);
   void Get_System_Name(char *szName);
   void Get_System_Pos(zVector3 &vPos);
   void Get_System_Vel(zVector3 &vVel);
   void Get_System_Moving(bool &bMoving);
   void Get_System_Age(float &fAge);
   void Get_System_MaxParticles(DWORD &dwMaxParticles);
   
   void Get_Emitter_EmitPerSec(float &fEmit);
   void Get_Emitter_NumParticle(DWORD &dwNumParticles);
   void Get_Emitter_Supress(bool &bSupress);
   void Get_Emitter_Inactive(bool &bNewPar);

   void Get_Par_PosVar(zVector3 &vPosVar);
   void Get_Par_Vel(zVector3 &vVelStart);
   void Get_Par_VelVar(zVector3 &vVelVar);
   void Get_Par_Force(zVector3 &vForceStart);
   void Get_Par_ForceVar(zVector3 &vForceVar);
   void Get_Par_SizeStart(float &fSizeStart);
   void Get_Par_SizeEnd(float &fSizeEnd);
   void Get_Par_SizeVar(float &fSizeVar);
   void Get_Par_ColorStart(D3DCOLORVALUE &clrStart);
   void Get_Par_ColorEnd(D3DCOLORVALUE &clrEnd);
   void Get_Par_ColorVar(D3DCOLORVALUE &clrVar);
   void Get_Par_Lifetime(float &fLifetime);
   void Get_Par_LifetimeVar(float &fLifetimeVar);  
};


// --------------------------------------------------------------------------------------------


#endif



#ifndef ZED_D3D_3DMODEL_H
#define ZED_D3D_3DMODEL_H


// ============================================================================================
// Defs
// ============================================================================================

// Die Definitionen der Strukturen stammen aus den original ID Software Sourcen !
const int zMAX_TRIANGLES   = 4096;
const int zMAX_VERTS       = 2048;
const int zMAX_FRAMES      = 512;
const int zMAX_MD2SKINS    = 32;
const int zMAX_SKINNAME    = 64;

//Zum identifizieren der Animation
struct SAnimation
{
   //Start und Ende der Animation
   int start, end;
   //Name
   std::string name;
   //An dieser Stelle ist die Animation gerade.
   //Wenn sie grer gleich als end ist, wird
   //sie wieder auf start gesetzt
   float cur;
   //add wird immer zu cur hinzugefgt, um die
   //animation was zu verlangsamen
   float add;
};

struct make_index_list
{
   int a, b, c;
   float a_s, a_t,
         b_s, b_t,
         c_s, c_t;
};

struct make_vertex_list
{
   float x, y, z;
};

struct make_frame_list
{
   make_vertex_list *vertex;
};

typedef unsigned char byte;

struct vec3_t
{
   float v[3];
};

struct dstvert_t
{
   //s und t sind die Positionen (x,y) auf dem Skin,
   //muss sein das bei Animationen die Position des
   //Skins wechseln kann
   short s;
   short t;
};

struct dtriangle_t
{
   short index_xyz[3];
   short index_st[3];
};

struct dtrivertx_t
{
   byte v[3];
   byte lightnormalindex;
};

//Hiermit werden die Frames identifiziert
struct daliasframe_t
{
   float scale[3];
   float translate[3];
   char name[16];
   dtrivertx_t verts[1];
};

struct SMD2Header
{
   int ident;
   int version;

   int skinwidth;       //Breite und Hhe des Skins
   int skinheight;      //Muss durch 8 teilbar sein

   int framesize;

   int num_skins;       //Anzahl der Skintexturen
   int num_xyz;
   int num_st;
   int num_tris;        //Anzahl der 3Ecke
   int num_glcmds;
   int num_frames;      //Anzahl der Frames

   //Verschiednene Offsetpositionen
   int ofs_skins;
   int ofs_st;
   int ofs_tris;
   int ofs_frames;
   int ofs_glcmds; 
   int ofs_end;
};

typedef struct
{
   vec3_t  v;
   int     lightnormalindex;
} trivert_t;

typedef struct
{
   vec3_t     mins, maxs;
   char       name[16];
   trivert_t  v[zMAX_VERTS];
} frame_t;

// Der Modelvertex, wie wir ihn fr D3D Vertexbuffers verwenden
struct zVertex_Md2Model 
{
   float x,y,z;
   DWORD diffuse;
   float tu,tv;
};

#define D3DFVF_MD2 ( D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_TEX1 | D3DFVF_TEXCOORDSIZE2(0) )

struct ZAPI SMesh
{
   std::vector <zVertex_Md2Model> vertex;
};

// --------------------------------------------------------------------------------------------

// ============================================================================================
// Klasse zMd2Model
// ============================================================================================

class ZAPI zMd2Model
{
private:
   make_index_list*  m_index_list;
   make_frame_list*  m_frame_list;
   long              m_frames, m_vertices, m_triangles;
   //Hier sind die "3Ecksdaten" gespeichert
   SMesh             m_data [zMAX_FRAMES];
   //Wird intern von Load aufgerufen
   int               Init();
public:
   //Siehe Beschreibung von SAnimation
   //DIESER VECTOR MUSS EXTERN NOCH AUSGEFLLT WERDEN !!!
   std::vector <SAnimation> m_anim;

   zMd2Model();
   ~zMd2Model();

   //Ldt die Datei und macht alle ntigen Initialisierungen
   BOOL Load(char* );
   //Lscht das Model und macht Speicher frei
   void Destroy();
   //Zeichnet Frame Number frame
   BOOL Render(int frame);
   //Spielt die Animation mit dem Namen name ab
   //und aktualisiert sie
   //Wenn advance=TRUE ist wird mit der ANimation
   //fortgefahren, ansonsten bleibt sie auf der
   //aktuellen Stelle stehen
   BOOL Render(char* name, BOOL advance=TRUE);

   inline int GetFrameCount()     { return m_frames;    }
   inline int GetVertexCount()    { return m_vertices;  }
   inline int GetTriangleCount()  { return m_triangles; }
};

// --------------------------------------------------------------------------------------------


#endif



// --------------------------------------------------------------------------------------------
// FILE_END
#endif
