// ============================================================================================
// Datei:      zhelp.h
//
// Desc:       Helper-Funcs fr Zed4D (Kapselung von Funcs die (noch) nicht untersttzt werden
//
// Notes:      ---
//             
// Coder:      Kongo
// Stand/Ver:  18.03.2002/*.*
// ============================================================================================

#ifndef ZED4D_HELPER_H
#define ZED4D_HELPER_H

/* Funktionen die mit _btest, _htest, _httest und _bttest berprft werden, werden bei Angabe
 * von WRITEFUNC vor dem Aufruf in die debug.txt geschrieben
 * zB: _btest(pDev->SetTexture(0,pTex)); schreibt in debug.txt: "pDev->SetTexture(0,pTex)"
 */
//#define WRITEFUNC

extern bool     bcheck;
extern HRESULT  hrcheck;


// ============================================================================================
// Defs
// ============================================================================================

/* Write File */
#define _write(x) EM.Write(x)
#define _writefunc(x) EM.Write(#x"\n")
#define _writefuncexec(x) EM.Write(#x"\n"); (x)

/* Error Checking */

#ifdef WRITEFUNC

#define _btest(x)     _writefunc(x); if ( !(x) )        return false;
#define _htest(x)     _writefunc(x); if ( FAILED(x) )   return false;
#define _bttest(x,y)  _writefunc(x); if ( !(x) )        { EM.Write(y); return false; }
#define _httest(x,y)  _writefunc(x); hrcheck=(x); if (FAILED(hrcheck)) {EM.Error(y,hrcheck); return false;}

#else

#define _btest(x)     if ( !(x) )        return false;
#define _htest(x)     if ( FAILED(x) )   return false;
#define _bttest(x,y)  if ( !(x) )        { EM.Write(y); return false; }
#define _httest(x,y)  hrcheck=(x); if (FAILED(hrcheck)) {EM.Error(y,hrcheck); return false;}

#endif

/* General */
#define _init(p)           {p = NULL;}
#define _release(p)        {if(p)p->Release(); p=NULL;}
#define _delete(p)         {if(p)delete p; p=NULL;}
#define _deletearray(p)    {if(p)delete []p; p=NULL;}
#define _zeromemory(p)     {ZeroMemory(&p,sizeof(p));}

/* Math */

#define _pi             3.14159265358979323846f
#define _pi_mul_2       6.28318530717958623200f
#define _pi_div_2       1.57079632679489655800f
#define _pi_div_4       0.78539816339744827900f

#define _tiny           0.000001f
#define _tiny2          0.0005f

#define _max(a,b)       (((a) > (b)) ? (a) : (b))
#define _min(a,b)       (((a) < (b)) ? (a) : (b))
#define _clamp(x, min, max)  x<min  ? min : x<max ? x : max
#define _abs(a)         ((a<0.0f) ? (-a) : (a))
#define _sgn(a)         ((a<0.0f) ? (-1) : (1))

#define _deg2rad(d)     ((d*_pi)/180.0f)
#define _rad2deg(r)     ((r*180.0f)/_pi)
#define _deg2rad_fast(d) ((d) * 0.01745329251994f)
#define _rad2deg_fast(r) ((r) * 57.29577951308232f)
#define _swap(x,y)      { float t=x; x=y; y=t; }

/* color */

#define _clrgb(r,g,b) (D3DCOLOR_XRGB((r),(g),(b)))
#define _clrgba(r,g,b,a) (D3DCOLOR_RGBA((r),(g),(b),(a)))
#define _clargb(a,r,g,b) (D3DCOLOR_ARGB((a),(r),(g),(b)))
#define _clrand()	(_rgb(RAND_INT(30,250), RAND_INT(20,250), RAND_INT(10,200)))

/* 3D */

struct D3DTLVERTEX {
   float x,y,z; // zVector3 v;
   float rhw;
   DWORD diffuse;
   DWORD specular;
   float tu,tv;
};

__forceinline D3DTLVERTEX createD3DTLVERTEX(float x,float y,float z,float rhw,DWORD dif,DWORD spec,float tu,float tv) {
   D3DTLVERTEX *v = new D3DTLVERTEX;
   v->x = x;
   v->y = y;
   v->z = z;
   v->rhw = rhw;
   v->diffuse = dif;
   v->specular = spec;
   v->tu = tu;
   v->tv = tv;
   return (*v);
}

struct D3DLVERTEX {
   float x,y,z; // zVector3 v;
   DWORD diffuse;
   DWORD specular;
   float tu,tv;
};

__forceinline D3DLVERTEX createD3DLVERTEX(float x,float y,float z, DWORD dif,DWORD spec,float tu,float tv) {
   D3DLVERTEX *v = new D3DLVERTEX;
   v->x = x;
   v->y = y;
   v->z = z;
   v->diffuse = dif;
   v->specular = spec;
   v->tu = tu;
   v->tv = tv;
   return (*v);
}

struct D3DVERTEX {
   float x,y,z;
   D3DVECTOR v; // zVector3 v;
   DWORD diffuse;
   float tu,tv;
};

__forceinline D3DVERTEX createD3DVERTEX(float x,float y,float z,zVector3 n,DWORD dif,float tu,float tv) {
   D3DVERTEX *v = new D3DVERTEX;
   v->x = x;
   v->y = y;
   v->z = z;
   v->v = n;
   v->diffuse = dif;
   v->tu = tu;
   v->tv = tv;
   return (*v);
}


#define D3DFVF_TLVERTEX (D3DFVF_XYZRHW|D3DFVF_DIFFUSE|D3DFVF_SPECULAR|D3DFVF_TEX1)
#define D3DFVF_LVERTEX (D3DFVF_XYZ|D3DFVF_DIFFUSE|D3DFVF_SPECULAR|D3DFVF_TEX1)
#define D3DFVF_VERTEX (D3DFVF_XYZ|D3DFVF_NORMAL|D3DFVF_DIFFUSE|D3DFVF_TEX1)

// --------------------------------------------------------------------------------------------


// ============================================================================================
// Funcs
// ============================================================================================

/* Math */

static inline float zm_acos(float x)
{
   if(x >  1.0f) x =  1.0f;
   if(x < -1.0f) x = -1.0f;
   return (float)acos(x);
}

// --------------------------------------------------------------------------------------------

static inline float zm_asin(float x)
{
   if(x >  1.0f) x =  1.0f;
   if(x < -1.0f) x = -1.0f;
   return (float)asin(x);
}

// --------------------------------------------------------------------------------------------

static inline float zm_sqrt(float x)
{
   if (x<0.0f) x=(float)0.0f;
   return (float) sqrt(x);
}

//-------------------------------------------------------------------
//  zm_fequal()
//  Return true if (f0==f1)
//-------------------------------------------------------------------
static inline bool zm_fequal(float f0, float f1, float tol) {
   float f = f0-f1;
   if ((f>(-tol)) && (f<tol)) return true;
   else                       return false;
}

//-------------------------------------------------------------------
//  zm_fless()
//  Return true if (f0<f1).
//-------------------------------------------------------------------
static inline bool zm_fless(float f0, float f1, float tol) {
   if ((f0-f1)<tol) return true;
   else             return false;
}

//-------------------------------------------------------------------
//  zm_fgreater()
//  Return true if (f0>f1)
//-------------------------------------------------------------------
static inline bool zm_fgreater(float f0, float f1, float tol) {
   if ((f0-f1)>tol) return true;
   else             return false;
}

// --------------------------------------------------------------------------------------------


// ============================================================================================
// IDirect3DDevice8 Helps
// ============================================================================================

#define _beginscene()     pDev->BeginScene()
#define _endscene()       pDev->EndScene()

#define _drawpu(primType, primCount, pointer, stride) \
                        pDev->DrawPrimitiveUP(primType,primCount,pointer,stride)
#define _drawp(primType, startVertex, numVertices) \
                        pDev->DrawPrimitive((primType),(startVertex),(numVertices))
#define _drawip(primType, MinIndex, NumVertices, StartIndex, PrimitiveCount) \
                        pDev->DrawIndexedPrimitive((primType),(MinIndex),(NumVertices),(StartIndex),(PrimitiveCount))

#define _setrs(state,val) pDev->SetRenderState((state),(val))

#define _setts(state,matrix) pDev->SetTransform((state),(matrix))

#define _settexst(stage,state,value) pDev->SetTextureStageState((stage),(state),(value))

#define _setmat(mat) pDev->SetMaterial(&(mat))

#define _setss(stage,stream,vertex) pDev->SetStreamSource((stage),(stream),sizeof(vertex))

#define _setvs(shader) pDev->SetVertexShader((shader))

#define _setind(ib,index) pDev->SetIndices(ib,index)

#define _clear(target,c,z,s) pDev->Clear(0L,NULL,(target),(c),(z),(s))

#define _present() pDev->Present(NULL,NULL,NULL,NULL)

// --------------------------------------------------------------------------------------------


#endif // ZED4D_HELPER_H
