/* ============================================================================================= *
 * file:    zstring.h
 * created: 2002-03-23
 * coder:   [kon]
 * ============================================================================================= */


/*************************************************************************************************/
/*************************************************************************************************/
// ***** Includes

#include <windows.h>
#include <stdio.h>
#include "zstring.h"


/*************************************************************************************************/
/*************************************************************************************************/
// ***** Funcs

zString::zString (const char* s)
{
   if ( !s ) {
      str = new char[1];
      str[0] = 0;
      return;
   }

   unsigned int uiLength = strlen(s);
   if ( uiLength == 0 ) {
      str = new char[1];
      str[0] = 0;
      return;
   }

   str = new char[uiLength+1];
   strcpy(str,s);
}

/*************************************************************************************************/

zString::zString (const zString& s)
{
   str = 0;
   *this = s;
}

/*************************************************************************************************/

zString::~zString ()
{
   delete[] str;
}

/*************************************************************************************************/

unsigned int zString::Length() const
{
   return strlen(str);
}

/*************************************************************************************************/

int zString::Find(const char *p) const
{
   char *pf = strstr(str, p);
   return (pf) ? (pf - str) : -1;
}

/*************************************************************************************************/

int zString::Find(const zString& s) const
{
   char *pf = strstr(str, s.str);
   return (pf) ? (pf - str) : -1;
}

/*************************************************************************************************/

void zString::MakeLower()
{
   if (str)
   {
      strlwr(str);
   }
}

/*************************************************************************************************/

void zString::TrimL()
{
   char* s = str;

   while ((*s == ' ') && (*s != '\0'))
      ++s;
   if (s != str) {
      char* newstr = new char[strlen(s)+1];
      strcpy(newstr,s);
      delete[] str;
      str = newstr;
   }
}

/*************************************************************************************************/

void zString::TrimR()
{
   char* s = &str[strlen(str)-1];

   while ((*s == ' ') && (s != str))
      --s;
   if (s != &str[strlen(str)-1]) {
      char* newstr = new char[s-str+2];
      strncpy(newstr,str,s-str+1);
      newstr[s-str+1] = '\0';
      delete[] str;
      str = newstr;
   }
}

/*************************************************************************************************/

void zString::TrimAll()
{
   TrimL();TrimR();
}

/*************************************************************************************************/

zString zString::SubL(int len) const
{
   zString ret;

   delete[] ret.str;
   ret.str = new char[len+1];
   strncpy(ret.str,str,len);
   ret.str[len] = '\0';

   return ret;
}

/*************************************************************************************************/

zString zString::SubR(int len) const
{
   zString ret;

   delete[] ret.str;
   ret.str = new char[len+1];
   strncpy(ret.str,str+strlen(str)-len/*-1*/,len);
   ret.str[len] = '\0';

   return ret;
}

/*************************************************************************************************/

zString zString::SubAll(int pos, int len) const
{
   zString ret;
   delete[] ret.str;
   ret.str = new char[len+1];
   strncpy(ret.str,str+pos,len);
   ret.str[len] = '\0';

   return ret;
}

/*************************************************************************************************/

inline bool scanred(char* reduce, char check)
{
   for (unsigned int i=0; i < strlen(reduce); i++) {
      if (reduce[i] == check)
         return true;
   }
   return false; 
}
/*-----------------------------------------------------------------------------------------------*/
void zString::Reduce(char* reducechars)
{
   int len = strlen(str);
   char* newstr = new char[len+1];

   for (int i=0; i<len+1; i++) newstr[i] = '\0';

   char* prevchar;
   char* checkchar;
   int ind=1;

   newstr[0] = str[0];
   for (i=1; i<=len; i++) {
      prevchar = &str[i-1];
      checkchar = &str[i];

      if ((*prevchar == *checkchar) && scanred(reducechars, *checkchar)) {
         // Gleiches Reduces Char hintereinander gefunden
      }
      else {
         newstr[ind++] = str[i];
      }
   }

   delete[]str;
   str = newstr;
}

/*************************************************************************************************/

int zString::Scan(char c) const
{
   for (unsigned int i=0; i < strlen(str); i++) {
      if (str[i] == c)
         return i;
   }
   return -1;
}

/*************************************************************************************************/

zString& zString::operator= (const zString& s)
{
   delete[] str;
   str = new char[strlen(s.str)+1];
   strcpy(str,s.str);
   return *this;
}

/*************************************************************************************************/

zString& zString::operator= (const char* s)
{
   delete[] str;
   str = new char[strlen(s)+1];
   strcpy(str,s);
   return *this;
}

/*************************************************************************************************/

bool zString::operator== (const zString& s) const
{
   return strlen(str) == strlen(s.str) && strcmp(str,s.str) == 0;
}



bool zString::operator!= (const zString& s) const
{
   return !operator==(s);
}

/*************************************************************************************************/

zString& zString::operator +=(const char *s)
{
   if (s) {
      if (str) {
         char *p = new char[strlen(str) + strlen(s) + 1];
         strcpy(p, str);
         strcat(p, s);
         delete str;
         str = p;
      }
      else {
         str = _strdup(s);
      }
   }
   return *this;
}

/*************************************************************************************************/

bool zString::operator== (const char* s) const
{
   return strlen(str) == strlen(s) && strcmp(str,s) == 0;
}

/*************************************************************************************************/

bool zString::operator!= (const char* s) const
{
   return !operator==(s);
}

/*************************************************************************************************/

zString zString::operator+ (const zString& s) const
{
   char *newstr = new char[strlen(str) + strlen(s.str) + 1];

   memcpy(newstr,str,strlen(str));
   memcpy(&newstr[strlen(str)],s.str,strlen(s.str));
   newstr[strlen(str) + strlen(s.str)] = '\0';
   return zString(newstr);
}

/*************************************************************************************************/

zString zString::operator+ (const char* s) const
{
   char *newstr = new char[strlen(str) + strlen(s) + 1];

   memcpy(newstr,str,strlen(str));
   memcpy(&newstr[strlen(str)],s,strlen(s));
   newstr[strlen(str) + strlen(s)] = '\0';
   return zString(newstr);
}

/*************************************************************************************************/

bool zString::operator<(const zString& s) const
{
   if (strlen(str) < strlen(s.str))
      return true;
   return false;
}

/*************************************************************************************************/

bool zString::operator<(const char * s) const
{
   if (strlen(str) < strlen(s))
      return true;
   return false;
}

/*************************************************************************************************/

bool zString::operator>(const zString& s) const
{
   if (strlen(str) > strlen(s.str))
      return true;
   return false;
}

/*************************************************************************************************/

bool zString::operator>(const char * s) const
{
   if (strlen(str) > strlen(s))
      return true;
   return false;
}

/*************************************************************************************************/


char& zString::operator[](int ind) const
{
   return str[ind];
}

/*************************************************************************************************/

void zString::Format(char* text, ...)
{
   va_list argptr;
   char	buf[32768];

   va_start (argptr,text);
   vsprintf (buf, text,argptr);
   va_end (argptr);

   delete[] str;
   str = new char[strlen(buf)+1];
   strcpy(str,buf);
}

/*************************************************************************************************/
